@@ -26,7 +26,6 @@ import org.apache.commons.collections.MapUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
 import org.apache.ranger.plugin.model.RangerPolicy;
 import org.apache.ranger.plugin.model.RangerServiceDef;
 import org.apache.ranger.plugin.model.RangerServiceResource;
@@ -34,16 +33,17 @@ import org.apache.ranger.plugin.model.RangerTag;
 import org.apache.ranger.plugin.model.validation.RangerServiceDefHelper;
 import org.apache.ranger.plugin.policyengine.RangerAccessRequest;
 import org.apache.ranger.plugin.policyengine.RangerAccessResource;
+import org.apache.ranger.plugin.policyengine.RangerAccessResourceImpl;
+import org.apache.ranger.plugin.policyengine.RangerResourceTrie;
 import org.apache.ranger.plugin.policyresourcematcher.RangerDefaultPolicyResourceMatcher;
 import org.apache.ranger.plugin.policyresourcematcher.RangerPolicyResourceMatcher;
 import org.apache.ranger.plugin.util.DownloadTrigger;
 import org.apache.ranger.plugin.util.DownloaderTask;
 import org.apache.ranger.plugin.service.RangerAuthContext;
-import org.apache.ranger.plugin.service.RangerBasePlugin;
 import org.apache.ranger.plugin.util.RangerAccessRequestUtil;
 import org.apache.ranger.plugin.util.RangerPerfTracer;
-import org.apache.ranger.plugin.util.RangerResourceTrie;
 import org.apache.ranger.plugin.util.RangerServiceNotFoundException;
+import org.apache.ranger.plugin.util.RangerServiceTagsDeltaUtil;
 import org.apache.ranger.plugin.util.ServiceTags;
 
 import java.io.File;
@@ -67,12 +67,13 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 	private static final Log LOG = LogFactory.getLog(RangerTagEnricher.class);
 
 	private static final Log PERF_CONTEXTENRICHER_INIT_LOG = RangerPerfTracer.getPerfLogger("contextenricher.init");
-	private static final Log PERF_TRIE_OP_LOG = RangerPerfTracer.getPerfLogger("resourcetrie.retrieval");
+	private static final Log PERF_TRIE_OP_LOG              = RangerPerfTracer.getPerfLogger("resourcetrie.retrieval");
+	private static final Log PERF_SET_SERVICETAGS_LOG      = RangerPerfTracer.getPerfLogger("tagenricher.setservicetags");
 
 
-	public static final String TAG_REFRESHER_POLLINGINTERVAL_OPTION = "tagRefresherPollingInterval";
-	public static final String TAG_RETRIEVER_CLASSNAME_OPTION       = "tagRetrieverClassName";
-	public static final String TAG_DISABLE_TRIE_PREFILTER_OPTION    = "disableTrieLookupPrefilter";
+	private static final String TAG_REFRESHER_POLLINGINTERVAL_OPTION = "tagRefresherPollingInterval";
+	private static final String TAG_RETRIEVER_CLASSNAME_OPTION       = "tagRetrieverClassName";
+	private static final String TAG_DISABLE_TRIE_PREFILTER_OPTION    = "disableTrieLookupPrefilter";
 
 	private RangerTagRefresher                 tagRefresher;
 	private RangerTagRetriever                 tagRetriever;
@@ -117,9 +118,10 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 
 			if (tagRetriever != null) {
 				String propertyPrefix    = "ranger.plugin." + serviceDef.getName();
-				disableCacheIfServiceNotFound = RangerConfiguration.getInstance().getBoolean(propertyPrefix + ".disable.cache.if.servicenotfound", true);
-				String cacheDir          = RangerConfiguration.getInstance().get(propertyPrefix + ".policy.cache.dir");
+				disableCacheIfServiceNotFound = getBooleanConfig(propertyPrefix + ".disable.cache.if.servicenotfound", true);
+				String cacheDir      = getConfig(propertyPrefix + ".policy.cache.dir", null);
 				String cacheFilename = String.format("%s_%s_tag.json", appId, serviceName);
+
 				cacheFilename = cacheFilename.replace(File.separatorChar,  '_');
 				cacheFilename = cacheFilename.replace(File.pathSeparatorChar,  '_');
 
@@ -127,9 +129,11 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 				tagRetriever.setServiceName(serviceName);
 				tagRetriever.setServiceDef(serviceDef);
 				tagRetriever.setAppId(appId);
+				tagRetriever.setPluginConfig(getPluginConfig());
 				tagRetriever.init(enricherDef.getEnricherOptions());
 
 				tagRefresher = new RangerTagRefresher(tagRetriever, this, -1L, tagDownloadQueue, cacheFile);
+				LOG.info("Created RangerTagRefresher Thread(" + tagRefresher.getName() + ")");
 
 				try {
 					tagRefresher.populateTags();
@@ -209,12 +213,12 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 	 * combinations in a service-def will be much smaller than the number of service-resources.
 	 */
 
-	static private class ResourceHierarchies {
+	static public class ResourceHierarchies {
 		private final Map<Collection<String>, Boolean> accessHierarchies    = new HashMap<>();
 		private final Map<Collection<String>, Boolean> dataMaskHierarchies  = new HashMap<>();
 		private final Map<Collection<String>, Boolean> rowFilterHierarchies = new HashMap<>();
 
-		public Boolean isValidHierarchy(int policyType, Collection<String> resourceKeys) {
+		Boolean isValidHierarchy(int policyType, Collection<String> resourceKeys) {
 			switch (policyType) {
 				case RangerPolicy.POLICY_TYPE_ACCESS:
 					return accessHierarchies.get(resourceKeys);
@@ -227,7 +231,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			}
 		}
 
-		public void addHierarchy(int policyType, Collection<String> resourceKeys, Boolean isValid) {
+		void addHierarchy(int policyType, Collection<String> resourceKeys, Boolean isValid) {
 			switch (policyType) {
 				case RangerPolicy.POLICY_TYPE_ACCESS:
 					accessHierarchies.put(resourceKeys, isValid);
@@ -246,115 +250,98 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 	}
 
 	public void setServiceTags(final ServiceTags serviceTags) {
-		if (serviceTags == null || CollectionUtils.isEmpty(serviceTags.getServiceResources())) {
-			LOG.info("ServiceTags is null or there are no tagged resources for service " + serviceName);
-			enrichedServiceTags = null;
-		} else {
-			List<RangerServiceResourceMatcher> resourceMatchers = new ArrayList<>();
+		boolean rebuildOnlyIndex = false;
+		setServiceTags(serviceTags, rebuildOnlyIndex);
+	}
 
-			RangerServiceDefHelper serviceDefHelper = new RangerServiceDefHelper(serviceDef, false);
+	protected void setServiceTags(final ServiceTags serviceTags, final boolean rebuildOnlyIndex) {
 
-			List<RangerServiceResource> serviceResources = serviceTags.getServiceResources();
-
-			ResourceHierarchies hierarchies = new ResourceHierarchies();
-
-			for (RangerServiceResource serviceResource : serviceResources) {
-				final Collection<String> resourceKeys = serviceResource.getResourceElements().keySet();
-
-				for (int policyType : RangerPolicy.POLICY_TYPES) {
-					Boolean isValidHierarchy = hierarchies.isValidHierarchy(policyType, resourceKeys);
-					if (isValidHierarchy == null) { // hierarchy not yet validated
-						isValidHierarchy = Boolean.FALSE;
-
-						for (List<RangerServiceDef.RangerResourceDef> hierarchy : serviceDefHelper.getResourceHierarchies(policyType)) {
-							if (serviceDefHelper.hierarchyHasAllResources(hierarchy, resourceKeys)) {
-								isValidHierarchy = Boolean.TRUE;
-
-								break;
-							}
-						}
-
-						hierarchies.addHierarchy(policyType, resourceKeys, isValidHierarchy);
-					}
-
-					if (isValidHierarchy) {
-						RangerDefaultPolicyResourceMatcher matcher = new RangerDefaultPolicyResourceMatcher();
-
-						matcher.setServiceDef(this.serviceDef);
-						matcher.setPolicyResources(serviceResource.getResourceElements(), policyType);
-
-						if (LOG.isDebugEnabled()) {
-							LOG.debug("RangerTagEnricher.setServiceTags() - Initializing matcher with (resource=" + serviceResource
-									+ ", serviceDef=" + this.serviceDef.getName() + ")");
-
-						}
-						matcher.setServiceDefHelper(serviceDefHelper);
-						matcher.init();
-
-						RangerServiceResourceMatcher serviceResourceMatcher = new RangerServiceResourceMatcher(serviceResource, matcher);
-						resourceMatchers.add(serviceResourceMatcher);
-					}
-				}
-			}
-
-
-			Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> serviceResourceTrie = null;
-
-			if (!disableTrieLookupPrefilter) {
-				serviceResourceTrie = new HashMap<>();
-
-				for (RangerServiceDef.RangerResourceDef resourceDef : serviceDef.getResources()) {
-					serviceResourceTrie.put(resourceDef.getName(), new RangerResourceTrie<RangerServiceResourceMatcher>(resourceDef, resourceMatchers));
-				}
-			}
-
-			Set<RangerTagForEval> tagsForEmptyResourceAndAnyAccess = new HashSet<>();
-			for (Map.Entry<Long, RangerTag> entry : serviceTags.getTags().entrySet()) {
-				tagsForEmptyResourceAndAnyAccess.add(new RangerTagForEval(entry.getValue(), RangerPolicyResourceMatcher.MatchType.DESCENDANT));
-			}
-
-			enrichedServiceTags = new EnrichedServiceTags(serviceTags, resourceMatchers, serviceResourceTrie, tagsForEmptyResourceAndAnyAccess);
-
-			Map<String, RangerBasePlugin> servicePluginMap = RangerBasePlugin.getServicePluginMap();
-			RangerBasePlugin plugin = servicePluginMap != null ? servicePluginMap.get(getServiceName()) : null;
-			if (plugin != null) {
-				RangerAuthContext currentAuthContext = plugin.getCurrentRangerAuthContext();
-				if (currentAuthContext != null) {
-					currentAuthContext.addOrReplaceRequestContextEnricher(this, enrichedServiceTags);
-					plugin.contextChanged();
-				}
-			}
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("==> RangerTagEnricher.setServiceTags(serviceTags=" + serviceTags + ", rebuildOnlyIndex=" + rebuildOnlyIndex + ")");
 		}
+
+		if (serviceTags == null) {
+			LOG.info("ServiceTags is null for service " + serviceName);
+			enrichedServiceTags = null;
+		} else  {
+			RangerPerfTracer perf = null;
+
+			if(RangerPerfTracer.isPerfTraceEnabled(PERF_SET_SERVICETAGS_LOG)) {
+				perf = RangerPerfTracer.getPerfTracer(PERF_SET_SERVICETAGS_LOG, "RangerTagEnricher.setServiceTags(newTagVersion=" + serviceTags.getTagVersion() + ",isDelta=" + serviceTags.getIsDelta() + ")");
+			}
+
+			if (!serviceTags.getIsDelta()) {
+				processServiceTags(serviceTags);
+			} else {
+				if (LOG.isDebugEnabled()) {
+					LOG.debug("Received service-tag deltas:" + serviceTags);
+				}
+
+				ServiceTags oldServiceTags = enrichedServiceTags != null ? enrichedServiceTags.getServiceTags() : new ServiceTags();
+				ServiceTags allServiceTags = rebuildOnlyIndex ? oldServiceTags : RangerServiceTagsDeltaUtil.applyDelta(oldServiceTags, serviceTags);
+
+				if (serviceTags.getTagsChangeExtent() == ServiceTags.TagsChangeExtent.NONE) {
+					if (LOG.isDebugEnabled()) {
+						LOG.debug("No change to service-tags other than version change");
+					}
+				} else {
+					if (serviceTags.getTagsChangeExtent() != ServiceTags.TagsChangeExtent.TAGS) {
+						processServiceTagDeltas(serviceTags, allServiceTags);
+					} else {
+						if (LOG.isDebugEnabled()) {
+							LOG.debug("Delta contains only tag attribute changes");
+						}
+						List<RangerServiceResourceMatcher>                            resourceMatchers    = enrichedServiceTags != null ? enrichedServiceTags.getServiceResourceMatchers() : new ArrayList<>();
+						Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> serviceResourceTrie = enrichedServiceTags != null ? enrichedServiceTags.getServiceResourceTrie() : new HashMap<>();
+						enrichedServiceTags = new EnrichedServiceTags(allServiceTags, resourceMatchers, serviceResourceTrie);
+					}
+				}
+			}
+			RangerPerfTracer.logAlways(perf);
+		}
+
+		setEnrichedServiceTagsInPlugin();
+
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("<== RangerTagEnricher.setServiceTags(serviceTags=" + serviceTags + ", rebuildOnlyIndex=" + rebuildOnlyIndex + ")");
+		}
+
 	}
 
 	protected Long getServiceTagsVersion() {
-		return enrichedServiceTags != null ? enrichedServiceTags.getServiceTags().getTagVersion() : null;
+		return enrichedServiceTags != null ? enrichedServiceTags.getServiceTags().getTagVersion() : -1L;
+	}
+
+	protected Long getResourceTrieVersion() {
+		return enrichedServiceTags != null ? enrichedServiceTags.getResourceTrieVersion() : -1L;
 	}
 
 	@Override
 	public boolean preCleanup() {
-		boolean ret = true;
-
 		if (LOG.isDebugEnabled()) {
 			LOG.debug("==> RangerTagEnricher.preCleanup()");
 		}
 
 		super.preCleanup();
 
+		Timer tagDownloadTimer = this.tagDownloadTimer;
+		this.tagDownloadTimer = null;
+
 		if (tagDownloadTimer != null) {
 			tagDownloadTimer.cancel();
-			tagDownloadTimer = null;
 		}
 
+		RangerTagRefresher tagRefresher = this.tagRefresher;
+		this.tagRefresher = null;
+
 		if (tagRefresher != null) {
 			tagRefresher.cleanup();
-			tagRefresher = null;
 		}
 
 		if (LOG.isDebugEnabled()) {
-			LOG.debug("<== RangerTagEnricher.preCleanup() : result=" + ret);
+			LOG.debug("<== RangerTagEnricher.preCleanup() : result=" + true);
 		}
-		return ret;
+		return true;
 	}
 
 	public void syncTagsWithAdmin(final DownloadTrigger token) throws InterruptedException {
@@ -362,6 +349,255 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 		token.waitForCompletion();
 	}
 
+	public EnrichedServiceTags getEnrichedServiceTags() {
+		return enrichedServiceTags;
+	}
+
+	private void processServiceTags(ServiceTags serviceTags) {
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("Processing all service-tags");
+		}
+
+		boolean isInError = false;
+
+		if (CollectionUtils.isEmpty(serviceTags.getServiceResources())) {
+			LOG.info("There are no tagged resources for service " + serviceName);
+			enrichedServiceTags = null;
+		} else {
+
+			RangerServiceDefHelper serviceDefHelper = new RangerServiceDefHelper(serviceDef, false);
+			ResourceHierarchies    hierarchies      = new ResourceHierarchies();
+
+			List<RangerServiceResourceMatcher> resourceMatchers = new ArrayList<>();
+			List<RangerServiceResource>        serviceResources = serviceTags.getServiceResources();
+
+			for (RangerServiceResource serviceResource : serviceResources) {
+				RangerServiceResourceMatcher serviceResourceMatcher = createRangerServiceResourceMatcher(serviceResource, serviceDefHelper, hierarchies);
+				if (serviceResourceMatcher != null) {
+					resourceMatchers.add(serviceResourceMatcher);
+				} else {
+					LOG.error("Could not create service-resource-matcher for service-resource:[" + serviceResource + "]");
+					isInError = true;
+					break;
+				}
+			}
+
+			if (isInError) {
+				serviceTags.setTagVersion(-1L);
+				LOG.error("Error in processing tag-deltas. Will continue to use old tags");
+			} else {
+				Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> serviceResourceTrie = null;
+
+				if (!disableTrieLookupPrefilter) {
+					serviceResourceTrie = new HashMap<>();
+
+					for (RangerServiceDef.RangerResourceDef resourceDef : serviceDef.getResources()) {
+						serviceResourceTrie.put(resourceDef.getName(), new RangerResourceTrie<>(resourceDef, resourceMatchers));
+					}
+				}
+				enrichedServiceTags = new EnrichedServiceTags(serviceTags, resourceMatchers, serviceResourceTrie);
+			}
+		}
+	}
+
+	private void processServiceTagDeltas(ServiceTags deltas, ServiceTags allServiceTags) {
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("Delta contains changes other than tag attribute changes, [" + deltas.getTagsChangeExtent() + "]");
+		}
+
+		boolean isInError = false;
+
+		RangerServiceDefHelper serviceDefHelper = new RangerServiceDefHelper(serviceDef, false);
+		ResourceHierarchies    hierarchies      = new ResourceHierarchies();
+
+		List<RangerServiceResourceMatcher>                            resourceMatchers    = new ArrayList<>();
+		Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> serviceResourceTrie = new HashMap<>();
+
+		if (enrichedServiceTags != null) {
+			resourceMatchers.addAll(enrichedServiceTags.getServiceResourceMatchers());
+			for (Map.Entry<String, RangerResourceTrie<RangerServiceResourceMatcher>> entry : enrichedServiceTags.getServiceResourceTrie().entrySet()) {
+				RangerResourceTrie<RangerServiceResourceMatcher> resourceTrie = new RangerResourceTrie<>(entry.getValue());
+				serviceResourceTrie.put(entry.getKey(), resourceTrie);
+			}
+		}
+
+		List<RangerServiceResource> changedServiceResources = deltas.getServiceResources();
+
+		for (RangerServiceResource serviceResource : changedServiceResources) {
+
+			final boolean removedOldServiceResource = MapUtils.isEmpty(serviceResource.getResourceElements()) || removeOldServiceResource(serviceResource, resourceMatchers, serviceResourceTrie);
+			if (removedOldServiceResource) {
+
+				if (!StringUtils.isEmpty(serviceResource.getResourceSignature())) {
+
+					RangerServiceResourceMatcher resourceMatcher = createRangerServiceResourceMatcher(serviceResource, serviceDefHelper, hierarchies);
+
+					if (resourceMatcher != null) {
+						for (RangerServiceDef.RangerResourceDef resourceDef : serviceDef.getResources()) {
+
+							RangerResourceTrie<RangerServiceResourceMatcher> trie = serviceResourceTrie.get(resourceDef.getName());
+
+							if (trie != null) {
+								trie.add(serviceResource.getResourceElements().get(resourceDef.getName()), resourceMatcher);
+								if (LOG.isDebugEnabled()) {
+									LOG.debug("Added resource-matcher for service-resource:[" + serviceResource + "]");
+								}
+							} else {
+								trie = new RangerResourceTrie<>(resourceDef, Collections.singletonList(resourceMatcher));
+								serviceResourceTrie.put(resourceDef.getName(), trie);
+							}
+						}
+						resourceMatchers.add(resourceMatcher);
+					} else {
+						LOG.error("Could not create resource-matcher for resource: [" + serviceResource + "]. Should NOT happen!!");
+						LOG.error("Setting tagVersion to -1 to ensure that in the next download all tags are downloaded");
+						isInError = true;
+					}
+				} else {
+					if (LOG.isDebugEnabled()) {
+						LOG.debug("Service-resource:[id=" + serviceResource.getId() + "] is deleted as its resource-signature is empty. No need to create it!");
+					}
+				}
+			} else {
+				isInError = true;
+			}
+			if (isInError) {
+				break;
+			}
+		}
+		if (isInError) {
+			LOG.error("Error in processing tag-deltas. Will continue to use old tags");
+			deltas.setTagVersion(-1L);
+		} else {
+			for (Map.Entry<String, RangerResourceTrie<RangerServiceResourceMatcher>> entry : serviceResourceTrie.entrySet()) {
+				entry.getValue().wrapUpUpdate();
+			}
+			enrichedServiceTags = new EnrichedServiceTags(allServiceTags, resourceMatchers, serviceResourceTrie);
+		}
+
+	}
+
+	private boolean removeOldServiceResource(RangerServiceResource serviceResource, List<RangerServiceResourceMatcher> resourceMatchers, Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> resourceTries) {
+		boolean ret = true;
+
+		if (enrichedServiceTags != null) {
+
+			if (LOG.isDebugEnabled()) {
+				LOG.debug("Removing service-resource:[" + serviceResource + "] from trie-map");
+			}
+
+			// Remove existing serviceResource from the copy
+
+			RangerAccessResourceImpl accessResource = new RangerAccessResourceImpl();
+
+			for (Map.Entry<String, RangerPolicy.RangerPolicyResource> entry : serviceResource.getResourceElements().entrySet()) {
+				accessResource.setValue(entry.getKey(), entry.getValue());
+			}
+			if (LOG.isDebugEnabled()) {
+				LOG.debug("RangerAccessResource:[" + accessResource + "] created to represent service-resource[" + serviceResource + "] to find evaluators from trie-map");
+			}
+
+			List<RangerServiceResourceMatcher> oldMatchers = getEvaluators(accessResource, enrichedServiceTags);
+
+			if (LOG.isDebugEnabled()) {
+				LOG.debug("Found [" + oldMatchers.size() + "] matchers for service-resource[" + serviceResource + "]");
+			}
+
+			for (RangerServiceResourceMatcher matcher : oldMatchers) {
+
+				for (String resourceDefName : serviceResource.getResourceElements().keySet()) {
+					RangerResourceTrie<RangerServiceResourceMatcher> trie = resourceTries.get(resourceDefName);
+					if (trie != null) {
+						trie.delete(serviceResource.getResourceElements().get(resourceDefName), matcher);
+					} else {
+						LOG.error("Cannot find resourceDef with name:[" + resourceDefName + "]. Should NOT happen!!");
+						LOG.error("Setting tagVersion to -1 to ensure that in the next download all tags are downloaded");
+						ret = false;
+						break;
+					}
+				}
+			}
+
+			// Remove old resource matchers
+			if (ret) {
+				resourceMatchers.removeAll(oldMatchers);
+
+				if (LOG.isDebugEnabled()) {
+					LOG.debug("Found and removed [" + oldMatchers.size() + "] matchers for service-resource[" + serviceResource + "] from trie-map");
+				}
+			}
+		}
+		return ret;
+	}
+
+	static public RangerServiceResourceMatcher createRangerServiceResourceMatcher(RangerServiceResource serviceResource, RangerServiceDefHelper serviceDefHelper, ResourceHierarchies hierarchies) {
+
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("==> createRangerServiceResourceMatcher(serviceResource=" + serviceResource + ")");
+		}
+
+		RangerServiceResourceMatcher ret = null;
+
+		final Collection<String> resourceKeys = serviceResource.getResourceElements().keySet();
+
+		for (int policyType : RangerPolicy.POLICY_TYPES) {
+			Boolean isValidHierarchy = hierarchies.isValidHierarchy(policyType, resourceKeys);
+			if (isValidHierarchy == null) { // hierarchy not yet validated
+				isValidHierarchy = Boolean.FALSE;
+
+				for (List<RangerServiceDef.RangerResourceDef> hierarchy : serviceDefHelper.getResourceHierarchies(policyType)) {
+					if (serviceDefHelper.hierarchyHasAllResources(hierarchy, resourceKeys)) {
+						isValidHierarchy = Boolean.TRUE;
+
+						break;
+					}
+				}
+
+				hierarchies.addHierarchy(policyType, resourceKeys, isValidHierarchy);
+			}
+
+			if (isValidHierarchy) {
+				RangerDefaultPolicyResourceMatcher matcher = new RangerDefaultPolicyResourceMatcher();
+
+				matcher.setServiceDef(serviceDefHelper.getServiceDef());
+				matcher.setPolicyResources(serviceResource.getResourceElements(), policyType);
+
+				if (LOG.isDebugEnabled()) {
+					LOG.debug("RangerTagEnricher.setServiceTags() - Initializing matcher with (resource=" + serviceResource
+							+ ", serviceDef=" + serviceDefHelper.getServiceDef() + ")");
+
+				}
+				matcher.setServiceDefHelper(serviceDefHelper);
+				matcher.init();
+
+				ret = new RangerServiceResourceMatcher(serviceResource, matcher);
+				break;
+			}
+		}
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("<== createRangerServiceResourceMatcher(serviceResource=" + serviceResource + ") : [" + ret + "]");
+		}
+		return ret;
+
+	}
+
+	private void setEnrichedServiceTagsInPlugin() {
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("==> setEnrichedServiceTagsInPlugin()");
+		}
+
+		RangerAuthContext authContext = getAuthContext();
+
+		if (authContext != null) {
+			authContext.addOrReplaceRequestContextEnricher(this, enrichedServiceTags);
+
+			notifyAuthContextChanged();
+		}
+
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("<== setEnrichedServiceTagsInPlugin()");
+		}
+	}
 
 	private Set<RangerTagForEval> findMatchingTags(final RangerAccessRequest request, EnrichedServiceTags dataStore) {
 		if (LOG.isDebugEnabled()) {
@@ -429,23 +665,24 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 		if(LOG.isDebugEnabled()) {
 			LOG.debug("==> RangerTagEnricher.getEvaluators(" + (resource != null ? resource.getAsString() : null) + ")");
 		}
-
-		List<RangerServiceResourceMatcher> ret = null;
+		List<RangerServiceResourceMatcher>  ret        = Collections.EMPTY_LIST;
 
 		final Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> serviceResourceTrie = enrichedServiceTags.getServiceResourceTrie();
 
 		if (resource == null || resource.getKeys() == null || resource.getKeys().isEmpty() || serviceResourceTrie == null) {
 			ret = enrichedServiceTags.getServiceResourceMatchers();
 		} else {
+			Set<RangerServiceResourceMatcher> evaluators = null;
+
 			RangerPerfTracer perf = null;
 
-			if(RangerPerfTracer.isPerfTraceEnabled(PERF_TRIE_OP_LOG)) {
+			if (RangerPerfTracer.isPerfTraceEnabled(PERF_TRIE_OP_LOG)) {
 				perf = RangerPerfTracer.getPerfTracer(PERF_TRIE_OP_LOG, "RangerTagEnricher.getEvaluators(resource=" + resource.getAsString() + ")");
 			}
 
-			Set<String> resourceKeys = resource.getKeys();
-			List<List<RangerServiceResourceMatcher>> serviceResourceMatchersList = null;
-			List<RangerServiceResourceMatcher> smallestList = null;
+			Set<String>                             resourceKeys = resource.getKeys();
+			List<Set<RangerServiceResourceMatcher>> serviceResourceMatchersList = null;
+			Set<RangerServiceResourceMatcher>       smallestList = null;
 
 			if (CollectionUtils.isNotEmpty(resourceKeys)) {
 
@@ -456,9 +693,9 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 						continue;
 					}
 
-					List<RangerServiceResourceMatcher> serviceResourceMatchers = trie.getEvaluatorsForResource(resource.getValue(resourceName));
+					Set<RangerServiceResourceMatcher> serviceResourceMatchers = trie.getEvaluatorsForResource(resource.getValue(resourceName));
 
-					if (CollectionUtils.isEmpty(serviceResourceMatchers)) { // no policies for this resource, bail out
+					if (CollectionUtils.isEmpty(serviceResourceMatchers)) { // no tags for this resource, bail out
 						serviceResourceMatchersList = null;
 						smallestList = null;
 						break;
@@ -479,26 +716,27 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 					}
 				}
 				if (serviceResourceMatchersList != null) {
-					ret = new ArrayList<>(smallestList);
-					for (List<RangerServiceResourceMatcher> serviceResourceMatchers : serviceResourceMatchersList) {
+					evaluators = new HashSet<>(smallestList);
+					for (Set<RangerServiceResourceMatcher> serviceResourceMatchers : serviceResourceMatchersList) {
 						if (serviceResourceMatchers != smallestList) {
-							// remove policies from ret that are not in serviceResourceMatchers
-							ret.retainAll(serviceResourceMatchers);
-							if (CollectionUtils.isEmpty(ret)) { // if no policy exists, bail out and return empty list
-								ret = null;
+							// remove other serviceResourceMatchers from ret that are not in serviceResourceMatchers
+							evaluators.retainAll(serviceResourceMatchers);
+							if (CollectionUtils.isEmpty(evaluators)) { // if no policy exists, bail out and return empty list
+								evaluators = null;
 								break;
 							}
 						}
 					}
 				} else {
-					ret = smallestList;
+					evaluators = smallestList;
 				}
 			}
-			RangerPerfTracer.logAlways(perf);
-		}
 
-		if(ret == null) {
-			ret = Collections.emptyList();
+			if (evaluators != null) {
+				ret = new ArrayList<>(evaluators);
+			}
+
+			RangerPerfTracer.logAlways(perf);
 		}
 
 		if(LOG.isDebugEnabled()) {
@@ -511,10 +749,13 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 	private static Set<RangerTagForEval> getTagsForServiceResource(final ServiceTags serviceTags, final RangerServiceResource serviceResource, final RangerPolicyResourceMatcher.MatchType matchType) {
 		Set<RangerTagForEval> ret = new HashSet<>();
 
-		final Long resourceId = serviceResource.getId();
-
+		final Long resourceId                        = serviceResource.getId();
 		final Map<Long, List<Long>> resourceToTagIds = serviceTags.getResourceToTagIds();
-		final Map<Long, RangerTag> tags = serviceTags.getTags();
+		final Map<Long, RangerTag> tags              = serviceTags.getTags();
+
+		if (LOG.isDebugEnabled()) {
+			LOG.debug("Looking for tags for resource-id:[" + resourceId + "] in serviceTags:[" + serviceTags + "]");
+		}
 
 		if (resourceId != null && MapUtils.isNotEmpty(resourceToTagIds) && MapUtils.isNotEmpty(tags)) {
 
@@ -530,29 +771,47 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 						ret.add(new RangerTagForEval(tag, matchType));
 					}
 				}
+			} else {
+				if (LOG.isDebugEnabled()) {
+					LOG.debug("No tags mapping found for resource:[" + resourceId + "]");
+				}
+			}
+		} else {
+			if (LOG.isDebugEnabled()) {
+				LOG.debug("resourceId is null or resourceToTagTds mapping is null or tags mapping is null!");
 			}
 		}
 
 		return ret;
 	}
 
-	static private final class EnrichedServiceTags {
-		final private ServiceTags                        serviceTags;
-		final private List<RangerServiceResourceMatcher> serviceResourceMatchers;
+	static public final class EnrichedServiceTags {
+		final private ServiceTags                                                      serviceTags;
+		final private List<RangerServiceResourceMatcher>                               serviceResourceMatchers;
 		final private Map<String, RangerResourceTrie<RangerServiceResourceMatcher>>    serviceResourceTrie;
-		final private Set<RangerTagForEval>              tagsForEmptyResourceAndAnyAccess; // Used only when accessed resource is empty and access type is 'any'
+		final private Set<RangerTagForEval>                                            tagsForEmptyResourceAndAnyAccess; // Used only when accessed resource is empty and access type is 'any'
+		final private Long                                                             resourceTrieVersion;
 
-		EnrichedServiceTags(ServiceTags serviceTags, List<RangerServiceResourceMatcher> serviceResourceMatchers,
-							Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> serviceResourceTrie, Set<RangerTagForEval> tagsForEmptyResourceAndAnyAccess) {
-			this.serviceTags             = serviceTags;
-			this.serviceResourceMatchers = serviceResourceMatchers;
-			this.serviceResourceTrie     = serviceResourceTrie;
-			this.tagsForEmptyResourceAndAnyAccess          = tagsForEmptyResourceAndAnyAccess;
+		EnrichedServiceTags(ServiceTags serviceTags, List<RangerServiceResourceMatcher> serviceResourceMatchers, Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> serviceResourceTrie) {
+			this.serviceTags                      = serviceTags;
+			this.serviceResourceMatchers          = serviceResourceMatchers;
+			this.serviceResourceTrie              = serviceResourceTrie;
+			this.tagsForEmptyResourceAndAnyAccess = createTagsForEmptyResourceAndAnyAccess();
+			this.resourceTrieVersion              = serviceTags.getTagVersion();
+		}
+		public ServiceTags                                                   getServiceTags() {return serviceTags;}
+		public List<RangerServiceResourceMatcher>                            getServiceResourceMatchers() { return serviceResourceMatchers;}
+		public Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> getServiceResourceTrie() { return serviceResourceTrie;}
+		public Long                                                          getResourceTrieVersion() { return resourceTrieVersion;}
+		public Set<RangerTagForEval>                                         getTagsForEmptyResourceAndAnyAccess() { return tagsForEmptyResourceAndAnyAccess;}
+
+		private Set<RangerTagForEval> createTagsForEmptyResourceAndAnyAccess() {
+			Set<RangerTagForEval> tagsForEmptyResourceAndAnyAccess = new HashSet<>();
+			for (Map.Entry<Long, RangerTag> entry : serviceTags.getTags().entrySet()) {
+				tagsForEmptyResourceAndAnyAccess.add(new RangerTagForEval(entry.getValue(), RangerPolicyResourceMatcher.MatchType.DESCENDANT));
+			}
+			return tagsForEmptyResourceAndAnyAccess;
 		}
-		ServiceTags getServiceTags() {return serviceTags;}
-		List<RangerServiceResourceMatcher> getServiceResourceMatchers() { return serviceResourceMatchers;}
-		Map<String, RangerResourceTrie<RangerServiceResourceMatcher>> getServiceResourceTrie() { return serviceResourceTrie;}
-		Set<RangerTagForEval> getTagsForEmptyResourceAndAnyAccess() { return tagsForEmptyResourceAndAnyAccess;}
 	}
 
 	static class RangerTagRefresher extends Thread {
@@ -560,7 +819,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 
 		private final RangerTagRetriever tagRetriever;
 		private final RangerTagEnricher tagEnricher;
-		private long lastKnownVersion = -1L;
+		private long lastKnownVersion;
 		private final BlockingQueue<DownloadTrigger> tagDownloadQueue;
 		private long lastActivationTimeInMillis;
 
@@ -579,6 +838,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			} catch(Throwable excp) {
 				LOG.fatal("failed to create GsonBuilder object", excp);
 			}
+			setName("RangerTagRefresher(serviceName=" + tagRetriever.getServiceName() + ")-" + getId());
 		}
 
 		public long getLastActivationTimeInMillis() {
@@ -597,22 +857,26 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			}
 
 			while (true) {
+				DownloadTrigger trigger = null;
 
 				try {
 					RangerPerfTracer perf = null;
 
 					if(RangerPerfTracer.isPerfTraceEnabled(PERF_CONTEXTENRICHER_INIT_LOG)) {
-						perf = RangerPerfTracer.getPerfTracer(PERF_CONTEXTENRICHER_INIT_LOG, "RangerTagRefresher.populateTags(serviceName=" + tagRetriever.getServiceName() + ",lastKnownVersion=" + lastKnownVersion + ")");
+						perf = RangerPerfTracer.getPerfTracer(PERF_CONTEXTENRICHER_INIT_LOG, "RangerTagRefresher(" + getName() + ").populateTags(lastKnownVersion=" + lastKnownVersion + ")");
 					}
-					DownloadTrigger trigger = tagDownloadQueue.take();
+					trigger = tagDownloadQueue.take();
 					populateTags();
-					trigger.signalCompletion();
 
 					RangerPerfTracer.log(perf);
 
 				} catch (InterruptedException excp) {
-					LOG.debug("RangerTagRefresher().run() : interrupted! Exiting thread", excp);
+					LOG.debug("RangerTagRefresher(" + getName() + ").run(): Interrupted! Exiting thread", excp);
 					break;
+				} finally {
+					if (trigger != null) {
+						trigger.signalCompletion();
+					}
 				}
 			}
 
@@ -624,7 +888,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 		private void populateTags() throws InterruptedException {
 
 			if (tagEnricher != null) {
-				ServiceTags serviceTags = null;
+				ServiceTags serviceTags;
 
 				try {
 					serviceTags = tagRetriever.retrieveTags(lastKnownVersion, lastActivationTimeInMillis);
@@ -633,20 +897,23 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 						if (!hasProvidedTagsToReceiver) {
 							serviceTags = loadFromCache();
 						}
-					} else {
+					} else if (!serviceTags.getIsDelta()) {
 						saveToCache(serviceTags);
 					}
 
 					if (serviceTags != null) {
 						tagEnricher.setServiceTags(serviceTags);
-						LOG.info("RangerTagRefresher.populateTags() - Updated tags-cache to new version of tags, lastKnownVersion=" + lastKnownVersion + "; newVersion="
+						if (serviceTags.getIsDelta() && serviceTags.getTagVersion() != -1L) {
+							saveToCache(tagEnricher.enrichedServiceTags.serviceTags);
+						}
+						LOG.info("RangerTagRefresher(serviceName=" + tagRetriever.getServiceName() + ").populateTags() - Updated tags-cache to new version of tags, lastKnownVersion=" + lastKnownVersion + "; newVersion="
 								+ (serviceTags.getTagVersion() == null ? -1L : serviceTags.getTagVersion()));
 						hasProvidedTagsToReceiver = true;
 						lastKnownVersion = serviceTags.getTagVersion() == null ? -1L : serviceTags.getTagVersion();
 						setLastActivationTimeInMillis(System.currentTimeMillis());
 					} else {
 						if (LOG.isDebugEnabled()) {
-							LOG.debug("RangerTagRefresher.populateTags() - No need to update tags-cache. lastKnownVersion=" + lastKnownVersion);
+							LOG.debug("RangerTagRefresher(serviceName=" + tagRetriever.getServiceName() + ").populateTags() - No need to update tags-cache. lastKnownVersion=" + lastKnownVersion);
 						}
 					}
 				} catch (RangerServiceNotFoundException snfe) {
@@ -662,11 +929,11 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 				} catch (InterruptedException interruptedException) {
 					throw interruptedException;
 				} catch (Exception e) {
-					LOG.error("Encountered unexpected exception. Ignoring", e);
+					LOG.error("RangerTagRefresher(serviceName=" + tagRetriever.getServiceName() + ").populateTags(): Encountered unexpected exception. Ignoring", e);
 				}
 
 			} else {
-				LOG.error("RangerTagRefresher.populateTags() - no tag receiver to update tag-cache");
+				LOG.error("RangerTagRefresher(serviceName=" + tagRetriever.getServiceName() + ".populateTags() - no tag receiver to update tag-cache");
 			}
 		}
 
@@ -686,7 +953,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			try {
 				super.start();
 			} catch (Exception excp) {
-				LOG.error("RangerTagRefresher.startRetriever() - failed to start, exception=" + excp);
+				LOG.error("RangerTagRefresher(" + getName() + ").startRetriever(): Failed to start, exception=" + excp);
 			}
 		}
 
@@ -695,10 +962,21 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 			if (super.isAlive()) {
 				super.interrupt();
 
-				try {
-					super.join();
-				} catch (InterruptedException excp) {
-					LOG.error("RangerTagRefresher(): error while waiting for thread to exit", excp);
+				boolean setInterrupted = false;
+				boolean isJoined = false;
+
+				while (!isJoined) {
+					try {
+						super.join();
+						isJoined = true;
+					} catch (InterruptedException excp) {
+						LOG.warn("RangerTagRefresher(" + getName() + ").stopRefresher(): Error while waiting for thread to exit", excp);
+						LOG.warn("Retrying Thread.join(). Current thread will be marked as 'interrupted' after Thread.join() returns");
+						setInterrupted = true;
+					}
+				}
+				if (setInterrupted) {
+					Thread.currentThread().interrupt();
 				}
 			}
 		}
@@ -776,7 +1054,7 @@ public class RangerTagEnricher extends RangerAbstractContextEnricher {
 					}
 				}
 			} else {
-				LOG.info("service-tags is null. Nothing to save in cache");
+				LOG.info("service-tags is null for service=" + tagRetriever.getServiceName() + ". Nothing to save in cache");
 			}
 
 			if (LOG.isDebugEnabled()) {
