@@ -41,8 +41,10 @@ import java.util.Map;
 public class RangerPluginInfo implements Serializable {
 	private static final long serialVersionUID = 1L;
 
-	public static final int ENTITY_TYPE_POLICIES = 0;
-	public static final int ENTITY_TYPE_TAGS     = 1;
+	public static final int ENTITY_TYPE_POLICIES 	= 0;
+	public static final int ENTITY_TYPE_TAGS     	= 1;
+	public static final int ENTITY_TYPE_ROLES	 	= 2;
+	public static final int ENTITY_TYPE_USERSTORE	= 3;
 
 	public static final String PLUGIN_INFO_POLICY_DOWNLOAD_TIME      = "policyDownloadTime";
 	public static final String PLUGIN_INFO_POLICY_DOWNLOADED_VERSION = "policyDownloadedVersion";
@@ -53,23 +55,38 @@ public class RangerPluginInfo implements Serializable {
 	public static final String PLUGIN_INFO_TAG_ACTIVATION_TIME       = "tagActivationTime";
 	public static final String PLUGIN_INFO_TAG_ACTIVE_VERSION        = "tagActiveVersion";
 
+	public static final String PLUGIN_INFO_ROLE_DOWNLOAD_TIME         = "roleDownloadTime";
+	public static final String PLUGIN_INFO_ROLE_DOWNLOADED_VERSION    = "roleDownloadedVersion";
+	public static final String PLUGIN_INFO_ROLE_ACTIVATION_TIME       = "roleActivationTime";
+	public static final String PLUGIN_INFO_ROLE_ACTIVE_VERSION        = "roleActiveVersion";
+
+	public static final String PLUGIN_INFO_USERSTORE_DOWNLOAD_TIME         = "userstoreDownloadTime";
+	public static final String PLUGIN_INFO_USERSTORE_DOWNLOADED_VERSION    = "userstoreDownloadedVersion";
+	public static final String PLUGIN_INFO_USERSTORE_ACTIVATION_TIME       = "userstoreActivationTime";
+	public static final String PLUGIN_INFO_USERSTORE_ACTIVE_VERSION        = "userstoreActiveVersion";
 
 	public static final String RANGER_ADMIN_LAST_POLICY_UPDATE_TIME  = "lastPolicyUpdateTime";
 	public static final String RANGER_ADMIN_LATEST_POLICY_VERSION    = "latestPolicyVersion";
 	public static final String RANGER_ADMIN_LAST_TAG_UPDATE_TIME     = "lastTagUpdateTime";
 	public static final String RANGER_ADMIN_LATEST_TAG_VERSION       = "latestTagVersion";
 
+	public static final String RANGER_ADMIN_CAPABILITIES             = "adminCapabilities";
+	public static final String PLUGIN_INFO_CAPABILITIES              = "pluginCapabilities";
+
 	private Long    id;
 	private Date    createTime;
 	private Date    updateTime;
 
 	private String serviceName;
-        private String serviceType;
+	private String serviceDisplayName;
+	private String serviceType;
+	private String serviceTypeDisplayName;
 	private String hostName;
 	private String appType;
 	private String ipAddress;
 	private Map<String, String> info;
 
+	//FIXME UNUSED
 	public RangerPluginInfo(Long id, Date createTime, Date updateTime, String serviceName, String appType, String hostName, String ipAddress, Map<String, String> info) {
 		super();
 
@@ -83,6 +100,7 @@ public class RangerPluginInfo implements Serializable {
 		setInfo(info);
 	}
 
+	//FIXME UNUSED
 	public RangerPluginInfo() {
 		this(null, null, null, null, null, null, null, null);
 	}
@@ -111,13 +129,21 @@ public class RangerPluginInfo implements Serializable {
 		this.updateTime = updateTime;
 	}
 
-        public String getServiceType() {
-                return serviceType;
-        }
+	public String getServiceType() {
+		return serviceType;
+	}
 
-        public void setServiceType(String serviceType) {
-                this.serviceType = serviceType;
-        }
+	public void setServiceType(String serviceType) {
+		this.serviceType = serviceType;
+	}
+
+	public String getServiceTypeDisplayName() {
+		return serviceTypeDisplayName;
+	}
+
+	public void setServiceTypeDisplayName(String serviceTypeDisplayName) {
+		this.serviceTypeDisplayName = serviceTypeDisplayName;
+	}
 
 	public String getServiceName() {
 		return serviceName;
@@ -127,6 +153,14 @@ public class RangerPluginInfo implements Serializable {
 		this.serviceName = serviceName;
 	}
 
+	public String getServiceDisplayName() {
+		return serviceDisplayName;
+	}
+
+	public void setServiceDisplayName(String serviceDisplayName) {
+		this.serviceDisplayName = serviceDisplayName;
+	}
+
 	public String getHostName() {
 		return hostName;
 	}
@@ -271,6 +305,125 @@ public class RangerPluginInfo implements Serializable {
 		return StringUtils.isNotBlank(updateTimeString) ? Long.valueOf(updateTimeString) : null;
 	}
 
+	@JsonIgnore
+	public void setRoleDownloadTime(Long roleDownloadTime) {
+		getInfo().put(PLUGIN_INFO_ROLE_DOWNLOAD_TIME, roleDownloadTime == null ? null : Long.toString(roleDownloadTime));
+	}
+
+	@JsonIgnore
+	public Long getRoleDownloadTime() {
+		String downloadTimeString = getInfo().get(PLUGIN_INFO_ROLE_DOWNLOAD_TIME);
+		return StringUtils.isNotBlank(downloadTimeString) ? Long.valueOf(downloadTimeString) : null;
+	}
+
+	@JsonIgnore
+	public void setRoleDownloadedVersion(Long roleDownloadedVersion) {
+		getInfo().put(PLUGIN_INFO_ROLE_DOWNLOADED_VERSION, roleDownloadedVersion == null ? null : Long.toString(roleDownloadedVersion));
+	}
+
+	@JsonIgnore
+	public Long getRoleDownloadedVersion() {
+		String downloadedVersionString = getInfo().get(PLUGIN_INFO_ROLE_DOWNLOADED_VERSION);
+		return StringUtils.isNotBlank(downloadedVersionString) ? Long.valueOf(downloadedVersionString) : null;
+	}
+
+	@JsonIgnore
+	public void setRoleActivationTime(Long roleActivationTime) {
+		getInfo().put(PLUGIN_INFO_ROLE_ACTIVATION_TIME, roleActivationTime == null ? null : Long.toString(roleActivationTime));
+	}
+
+	@JsonIgnore
+	public Long getRoleActivationTime() {
+		String activationTimeString = getInfo().get(PLUGIN_INFO_ROLE_ACTIVATION_TIME);
+		return StringUtils.isNotBlank(activationTimeString) ? Long.valueOf(activationTimeString) : null;
+	}
+
+	@JsonIgnore
+	public void setRoleActiveVersion(Long roleActiveVersion) {
+		getInfo().put(PLUGIN_INFO_ROLE_ACTIVE_VERSION, roleActiveVersion == null ? null : Long.toString(roleActiveVersion));
+	}
+
+	@JsonIgnore
+	public Long getRoleActiveVersion() {
+		String activeVersionString = getInfo().get(PLUGIN_INFO_POLICY_ACTIVE_VERSION);
+		return StringUtils.isNotBlank(activeVersionString) ? Long.valueOf(activeVersionString) : null;
+	}
+
+	@JsonIgnore
+	public void setUserStoreDownloadTime(Long userstoreDownloadTime) {
+		getInfo().put(PLUGIN_INFO_USERSTORE_DOWNLOAD_TIME, userstoreDownloadTime == null ? null : Long.toString(userstoreDownloadTime));
+	}
+
+	@JsonIgnore
+	public Long getUserStoreDownloadTime() {
+		String downloadTimeString = getInfo().get(PLUGIN_INFO_USERSTORE_DOWNLOAD_TIME);
+		return StringUtils.isNotBlank(downloadTimeString) ? Long.valueOf(downloadTimeString) : null;
+	}
+
+	@JsonIgnore
+	public void setUserStoreDownloadedVersion(Long userstoreDownloadedVersion) {
+		getInfo().put(PLUGIN_INFO_USERSTORE_DOWNLOADED_VERSION, userstoreDownloadedVersion == null ? null : Long.toString(userstoreDownloadedVersion));
+	}
+
+	@JsonIgnore
+	public Long getUserStoreDownloadedVersion() {
+		String downloadedVersionString = getInfo().get(PLUGIN_INFO_USERSTORE_DOWNLOADED_VERSION);
+		return StringUtils.isNotBlank(downloadedVersionString) ? Long.valueOf(downloadedVersionString) : null;
+	}
+
+	@JsonIgnore
+	public void setUserStoreActivationTime(Long userstoreActivationTime) {
+		getInfo().put(PLUGIN_INFO_USERSTORE_ACTIVATION_TIME, userstoreActivationTime == null ? null : Long.toString(userstoreActivationTime));
+	}
+
+	@JsonIgnore
+	public Long getUserStoreActivationTime() {
+		String activationTimeString = getInfo().get(PLUGIN_INFO_USERSTORE_ACTIVATION_TIME);
+		return StringUtils.isNotBlank(activationTimeString) ? Long.valueOf(activationTimeString) : null;
+	}
+
+	@JsonIgnore
+	public void setUserStoreActiveVersion(Long userstoreActiveVersion) {
+		getInfo().put(PLUGIN_INFO_USERSTORE_ACTIVE_VERSION, userstoreActiveVersion == null ? null : Long.toString(userstoreActiveVersion));
+	}
+
+	@JsonIgnore
+	public Long getUserStoreActiveVersion() {
+		String activeVersionString = getInfo().get(PLUGIN_INFO_USERSTORE_ACTIVE_VERSION);
+		return StringUtils.isNotBlank(activeVersionString) ? Long.valueOf(activeVersionString) : null;
+	}
+
+	@JsonIgnore
+	public void setPluginCapabilities(String capabilities) {
+		setCapabilities(PLUGIN_INFO_CAPABILITIES, capabilities);
+	}
+
+	@JsonIgnore
+	public String getPluginCapabilities() {
+		return getCapabilities(PLUGIN_INFO_CAPABILITIES);
+	}
+
+	@JsonIgnore
+	public void setAdminCapabilities(String capabilities) {
+		setCapabilities(RANGER_ADMIN_CAPABILITIES, capabilities);
+	}
+
+	@JsonIgnore
+	public String getAdminCapabilities() {
+		return getCapabilities(RANGER_ADMIN_CAPABILITIES);
+	}
+
+	@JsonIgnore
+	private void setCapabilities(String optionName, String capabilities) {
+		getInfo().put(optionName, capabilities == null ? null : capabilities);
+	}
+
+	@JsonIgnore
+	private String getCapabilities(String optionName) {
+		String capabilitiesString = getInfo().get(optionName);
+		return StringUtils.isNotBlank(capabilitiesString) ? capabilitiesString : null;
+	}
+
 	@Override
 	public String toString() {
 		StringBuilder sb = new StringBuilder();
@@ -287,7 +440,9 @@ public class RangerPluginInfo implements Serializable {
 		sb.append("createTime={").append(createTime).append("} ");
 		sb.append("updateTime={").append(updateTime).append("} ");
 		sb.append("serviceName={").append(serviceName).append("} ");
-                sb.append("serviceType={").append(serviceType).append("} ");
+		sb.append("serviceType={").append(serviceType).append("} ");
+		sb.append("serviceTypeDisplayName{").append(serviceTypeDisplayName).append("} ");
+		sb.append("serviceDisplayName={").append(serviceDisplayName).append("} ");
 		sb.append("hostName={").append(hostName).append("} ");
 		sb.append("appType={").append(appType).append("} ");
 		sb.append("ipAddress={").append(ipAddress).append("} ");
