@@ -25,6 +25,6 @@ import org.apache.ranger.plugin.model.RangerServiceDef;
 public interface RangerMutableResource extends RangerAccessResource {
 	void setOwnerUser(String ownerUser);
 
-	void setValue(String type, String value);
+	void setValue(String type, Object value);
 	void setServiceDef(RangerServiceDef serviceDef);
 }
