@@ -20,6 +20,8 @@
 package org.apache.ranger.plugin.policyengine;
 
 import org.apache.commons.lang.StringUtils;
+import org.apache.ranger.plugin.model.RangerPolicy;
+
 import org.codehaus.jackson.annotate.JsonAutoDetect;
 import org.codehaus.jackson.annotate.JsonIgnoreProperties;
 import org.codehaus.jackson.map.annotate.JsonSerialize;
@@ -38,7 +40,7 @@ import static org.apache.ranger.plugin.policyevaluator.RangerPolicyEvaluator.ACC
 public class RangerResourceACLs {
 	final private Map<String, Map<String, AccessResult>> userACLs  = new HashMap<>();
 	final private Map<String, Map<String, AccessResult>> groupACLs = new HashMap<>();
-
+	final private Map<String, Map<String, AccessResult>> roleACLs  = new HashMap<>();
 	public RangerResourceACLs() {
 	}
 
@@ -50,6 +52,8 @@ public class RangerResourceACLs {
 		return groupACLs;
 	}
 
+	public Map<String, Map<String, AccessResult>> getRoleACLs() { return roleACLs; }
+
 	public void finalizeAcls() {
 		Map<String, AccessResult>  publicGroupAccessInfo = groupACLs.get(RangerPolicyEngine.GROUP_PUBLIC);
 		if (publicGroupAccessInfo != null) {
@@ -82,9 +86,10 @@ public class RangerResourceACLs {
 		}
 		finalizeAcls(userACLs);
 		finalizeAcls(groupACLs);
+		finalizeAcls(roleACLs);
 	}
 
-	public void setUserAccessInfo(String userName, String accessType, Integer access) {
+	public void setUserAccessInfo(String userName, String accessType, Integer access, RangerPolicy policy) {
 		Map<String, AccessResult> userAccessInfo = userACLs.get(userName);
 
 		if (userAccessInfo == null) {
@@ -96,15 +101,16 @@ public class RangerResourceACLs {
 		AccessResult accessResult = userAccessInfo.get(accessType);
 
 		if (accessResult == null) {
-			accessResult = new AccessResult(access);
+			accessResult = new AccessResult(access, policy);
 
 			userAccessInfo.put(accessType, accessResult);
 		} else {
 			accessResult.setResult(access);
+			accessResult.setPolicy(policy);
 		}
 	}
 
-	public void setGroupAccessInfo(String groupName, String accessType, Integer access) {
+	public void setGroupAccessInfo(String groupName, String accessType, Integer access, RangerPolicy policy) {
 		Map<String, AccessResult> groupAccessInfo = groupACLs.get(groupName);
 
 		if (groupAccessInfo == null) {
@@ -116,11 +122,33 @@ public class RangerResourceACLs {
 		AccessResult accessResult = groupAccessInfo.get(accessType);
 
 		if (accessResult == null) {
-			accessResult = new AccessResult(access);
+			accessResult = new AccessResult(access, policy);
 
 			groupAccessInfo.put(accessType, accessResult);
 		} else {
 			accessResult.setResult(access);
+			accessResult.setPolicy(policy);
+		}
+	}
+
+	public void setRoleAccessInfo(String roleName, String accessType, Integer access, RangerPolicy policy) {
+		Map<String, AccessResult> roleAccessInfo = roleACLs.get(roleName);
+
+		if (roleAccessInfo == null) {
+			roleAccessInfo = new HashMap<>();
+
+			roleACLs.put(roleName, roleAccessInfo);
+		}
+
+		AccessResult accessResult = roleAccessInfo.get(accessType);
+
+		if (accessResult == null) {
+			accessResult = new AccessResult(access, policy);
+
+			roleAccessInfo.put(accessType, accessResult);
+		} else {
+			accessResult.setResult(access);
+			accessResult.setPolicy(policy);
 		}
 	}
 
@@ -129,26 +157,43 @@ public class RangerResourceACLs {
 		StringBuffer sb = new StringBuffer();
 
 		sb.append("{");
+
 		sb.append("UserACLs={");
 		for (Map.Entry<String, Map<String, AccessResult>> entry : userACLs.entrySet()) {
 			sb.append("user=").append(entry.getKey()).append(":");
 			sb.append("permissions={");
 			for (Map.Entry<String, AccessResult> permission : entry.getValue().entrySet()) {
 				sb.append("{Permission=").append(permission.getKey()).append(", value=").append(permission.getValue()).append("},");
+				sb.append("{RangerPolicyID=").append(permission.getValue().getPolicy().getId()).append("},");
 			}
 			sb.append("},");
 		}
 		sb.append("}");
+
 		sb.append(", GroupACLs={");
 		for (Map.Entry<String, Map<String, AccessResult>> entry : groupACLs.entrySet()) {
 			sb.append("group=").append(entry.getKey()).append(":");
 			sb.append("permissions={");
 			for (Map.Entry<String, AccessResult> permission : entry.getValue().entrySet()) {
 				sb.append("{Permission=").append(permission.getKey()).append(", value=").append(permission.getValue()).append("}, ");
+				sb.append("{RangerPolicy ID=").append(permission.getValue().getPolicy().getId()).append("},");
 			}
 			sb.append("},");
 		}
 		sb.append("}");
+
+		sb.append(", RoleACLs={");
+		for (Map.Entry<String, Map<String, AccessResult>> entry : roleACLs.entrySet()) {
+			sb.append("role=").append(entry.getKey()).append(":");
+			sb.append("permissions={");
+			for (Map.Entry<String, AccessResult> permission : entry.getValue().entrySet()) {
+				sb.append("{Permission=").append(permission.getKey()).append(", value=").append(permission.getValue()).append("}, ");
+				sb.append("{RangerPolicy ID=").append(permission.getValue().getPolicy().getId()).append("},");
+			}
+			sb.append("},");
+		}
+		sb.append("}");
+
 		sb.append("}");
 
 		return sb.toString();
@@ -179,19 +224,20 @@ public class RangerResourceACLs {
 	public static class AccessResult {
 		private int     result;
 		private boolean isFinal;
-
+		private RangerPolicy  policy;
 
 		public AccessResult() {
-			this(-1);
+			this(-1, null);
 		}
 
-		public AccessResult(int result) {
-			this(result, false);
+		public AccessResult(int result, RangerPolicy policy) {
+			this(result, false, policy);
 		}
 
-		public AccessResult(int result, boolean isFinal) {
+		public AccessResult(int result, boolean isFinal, RangerPolicy policy) {
 			setIsFinal(isFinal);
 			setResult(result);
+			setPolicy(policy);
 		}
 
 		public int getResult() { return result; }
@@ -210,6 +256,14 @@ public class RangerResourceACLs {
 
 		public void setIsFinal(boolean isFinal) { this.isFinal = isFinal; }
 
+		public RangerPolicy getPolicy() {
+			return policy;
+		}
+
+		public void setPolicy(RangerPolicy policy){
+			this.policy = policy;
+		}
+
 		@Override
 		public boolean equals(Object other) {
 			if (other == null)
