@@ -20,6 +20,7 @@
 package org.apache.ranger.plugin.policyengine;
 
 
+import org.apache.commons.lang.StringUtils;
 import org.apache.ranger.plugin.contextenricher.RangerTagForEval;
 import org.apache.ranger.plugin.model.RangerServiceDef;
 import org.apache.ranger.plugin.policyresourcematcher.RangerPolicyResourceMatcher;
@@ -34,6 +35,7 @@ public class RangerTagAccessRequest extends RangerAccessRequestImpl {
 		super.setResource(new RangerTagResource(resourceTag.getType(), tagServiceDef));
 		super.setUser(request.getUser());
 		super.setUserGroups(request.getUserGroups());
+		super.setUserRoles(request.getUserRoles());
 		super.setAction(request.getAction());
 		super.setAccessType(request.getAccessType());
 		super.setAccessTime(request.getAccessTime());
@@ -45,6 +47,11 @@ public class RangerTagAccessRequest extends RangerAccessRequestImpl {
 		RangerAccessRequestUtil.setCurrentResourceInContext(request.getContext(), request.getResource());
 		RangerAccessRequestUtil.setCurrentUserInContext(request.getContext(), request.getUser());
 
+		String owner = request.getResource() != null ? request.getResource().getOwnerUser() : null;
+
+		if (StringUtils.isNotEmpty(owner)) {
+			RangerAccessRequestUtil.setOwnerInContext(request.getContext(), owner);
+		}
 		super.setContext(requestContext);
 
 		super.setClientType(request.getClientType());
