@@ -19,7 +19,6 @@
 package org.apache.ranger.plugin.policyevaluator;
 
 import java.util.Collections;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
@@ -52,7 +51,6 @@ public class RangerDefaultPolicyItemEvaluator extends RangerAbstractPolicyItemEv
 
 	private boolean hasCurrentUser;
 	private boolean hasResourceOwner;
-	private boolean hasAllPerms;
 
 	public RangerDefaultPolicyItemEvaluator(RangerServiceDef serviceDef, RangerPolicy policy, RangerPolicyItem policyItem, int policyItemType, int policyItemIndex, RangerPolicyEngineOptions options) {
 		super(serviceDef, policy, policyItem, policyItemType, policyItemIndex, options);
@@ -63,26 +61,6 @@ public class RangerDefaultPolicyItemEvaluator extends RangerAbstractPolicyItemEv
 			LOG.debug("==> RangerDefaultPolicyItemEvaluator(policyId=" + policyId + ", policyItem=" + policyItem + ", serviceType=" + getServiceType() + ", conditionsDisabled=" + getConditionsDisabledOption() + ")");
 		}
 
-		Set<String> accessPerms    = new HashSet<String>();
-
-		List<RangerPolicy.RangerPolicyItemAccess> policyItemAccesses = policyItem.getAccesses();
-		for(RangerPolicy.RangerPolicyItemAccess policyItemAccess : policyItemAccesses) {
-
-			if (policyItemAccess.getIsAllowed()) {
-				accessPerms.add(policyItemAccess.getType());
-			}
-		}
-
-		hasAllPerms = true;
-		List<RangerServiceDef.RangerAccessTypeDef> serviceAccessTypes = serviceDef.getAccessTypes();
-		for (RangerServiceDef.RangerAccessTypeDef serviceAccessType : serviceAccessTypes) {
-			String serviceAccessTypeName = serviceAccessType.getName();
-			if (!accessPerms.contains(serviceAccessTypeName)) {
-				hasAllPerms = false;
-				break;
-			}
-		}
-
 		RangerCustomConditionEvaluator rangerCustomConditionEvaluator = new RangerCustomConditionEvaluator();
 
 		conditionEvaluators = rangerCustomConditionEvaluator.getPolicyItemConditionEvaluator(policy,policyItem,serviceDef,options,policyItemIndex);
@@ -119,25 +97,10 @@ public class RangerDefaultPolicyItemEvaluator extends RangerAbstractPolicyItemEv
 				} else if (CollectionUtils.isNotEmpty(policyItem.getAccesses())) {
 					boolean isAccessTypeMatched = false;
 
-					if (request.isAccessTypeAny()) {
-						if (getPolicyItemType() == POLICY_ITEM_TYPE_DENY || getPolicyItemType() == POLICY_ITEM_TYPE_DENY_EXCEPTIONS) {
-							if (hasAllPerms) {
-								isAccessTypeMatched = true;
-							}
-						} else {
-							for (RangerPolicy.RangerPolicyItemAccess access : policyItem.getAccesses()) {
-								if (access.getIsAllowed()) {
-									isAccessTypeMatched = true;
-									break;
-								}
-							}
-						}
-					} else {
-						for (RangerPolicy.RangerPolicyItemAccess access : policyItem.getAccesses()) {
-							if (access.getIsAllowed() && StringUtils.equalsIgnoreCase(access.getType(), request.getAccessType())) {
-								isAccessTypeMatched = true;
-								break;
-							}
+					for (RangerPolicy.RangerPolicyItemAccess access : policyItem.getAccesses()) {
+						if (access.getIsAllowed() && StringUtils.equalsIgnoreCase(access.getType(), request.getAccessType())) {
+							isAccessTypeMatched = true;
+							break;
 						}
 					}
 
@@ -160,9 +123,9 @@ public class RangerDefaultPolicyItemEvaluator extends RangerAbstractPolicyItemEv
 	}
 
 	@Override
-	public boolean matchUserGroup(String user, Set<String> userGroups, Set<String> roles) {
+	public boolean matchUserGroupAndOwner(String user, Set<String> userGroups, Set<String> roles, String owner) {
 		if(LOG.isDebugEnabled()) {
-			LOG.debug("==> RangerDefaultPolicyItemEvaluator.matchUserGroup(" + policyItem + ", " + user + ", " + userGroups + ", " + roles + ")");
+			LOG.debug("==> RangerDefaultPolicyItemEvaluator.matchUserGroup(" + policyItem + ", " + user + ", " + userGroups + ", " + roles + ", " + owner + ")");
 		}
 
 		boolean ret = false;
@@ -171,7 +134,6 @@ public class RangerDefaultPolicyItemEvaluator extends RangerAbstractPolicyItemEv
 			if(!ret && user != null && policyItem.getUsers() != null) {
 				ret = hasCurrentUser || policyItem.getUsers().contains(user);
 			}
-
 			if(!ret && userGroups != null && policyItem.getGroups() != null) {
 				ret = policyItem.getGroups().contains(RangerPolicyEngine.GROUP_PUBLIC) ||
 						!Collections.disjoint(policyItem.getGroups(), userGroups);
@@ -179,10 +141,13 @@ public class RangerDefaultPolicyItemEvaluator extends RangerAbstractPolicyItemEv
 			if (!ret && CollectionUtils.isNotEmpty(roles) && CollectionUtils.isNotEmpty(policyItem.getRoles())) {
 				ret = !Collections.disjoint(policyItem.getRoles(), roles);
 			}
+			if (!ret && hasResourceOwner) {
+				ret = user != null && user.equals(owner);
+			}
 		}
 
 		if(LOG.isDebugEnabled()) {
-			LOG.debug("<== RangerDefaultPolicyItemEvaluator.matchUserGroup(" + policyItem + ", " + user + ", " + userGroups + ", " + roles + "): " + ret);
+			LOG.debug("<== RangerDefaultPolicyItemEvaluator.matchUserGroup(" + policyItem + ", " + user + ", " + userGroups + ", " + roles + ", " + owner + "): " + ret);
 		}
 
 		return ret;
@@ -198,20 +163,15 @@ public class RangerDefaultPolicyItemEvaluator extends RangerAbstractPolicyItemEv
 		String user = request.getUser();
 		Set<String> userGroups = request.getUserGroups();
 
-		if (hasResourceOwner) {
-			RangerAccessResource accessedResource = request.getResource();
-			String resourceOwner = accessedResource != null ? accessedResource.getOwnerUser() : null;
+		RangerAccessResource accessedResource = request.getResource();
+		String resourceOwner = accessedResource != null ? accessedResource.getOwnerUser() : null;
 
-			if (user != null && resourceOwner != null && user.equals(resourceOwner)) {
-				ret = true;
-			}
-		}
 		if (!ret) {
 			Set<String> roles = null;
 			if (CollectionUtils.isNotEmpty(policyItem.getRoles())) {
 				roles = RangerAccessRequestUtil.getCurrentUserRolesFromContext(request.getContext());
 			}
-			ret = matchUserGroup(user, userGroups, roles);
+			ret = matchUserGroupAndOwner(user, userGroups, roles, resourceOwner);
 		}
 
 		if(LOG.isDebugEnabled()) {
