@@ -31,6 +31,10 @@ import java.security.NoSuchAlgorithmException;
 import java.security.SecureRandom;
 import java.security.UnrecoverableKeyException;
 import java.security.cert.CertificateException;
+import java.util.List;
+import java.util.Map;
+import java.util.Random;
+import java.util.Set;
 
 import javax.net.ssl.HostnameVerifier;
 import javax.net.ssl.KeyManager;
@@ -39,13 +43,13 @@ import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLSession;
 import javax.net.ssl.TrustManager;
 import javax.net.ssl.TrustManagerFactory;
+import javax.ws.rs.core.Cookie;
 
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.lang.Validate;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
-import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
 import org.apache.ranger.authorization.hadoop.utils.RangerCredentialProvider;
 import org.apache.ranger.authorization.utils.StringUtil;
 import org.codehaus.jackson.jaxrs.JacksonJsonProvider;
@@ -53,6 +57,8 @@ import org.codehaus.jackson.jaxrs.JacksonJsonProvider;
 import com.google.gson.Gson;
 import com.google.gson.GsonBuilder;
 import com.sun.jersey.api.client.Client;
+import com.sun.jersey.api.client.ClientHandlerException;
+import com.sun.jersey.api.client.ClientResponse;
 import com.sun.jersey.api.client.WebResource;
 import com.sun.jersey.api.client.config.ClientConfig;
 import com.sun.jersey.api.client.config.DefaultClientConfig;
@@ -96,23 +102,24 @@ public class RangerRESTClient {
 	private String mTrustStoreAlias;
 	private String mTrustStoreFile;
 	private String mTrustStoreType;
-
 	private Gson   gsonBuilder;
+	private int    mRestClientConnTimeOutMs;
+	private int    mRestClientReadTimeOutMs;
+	private int    lastKnownActiveUrlIndex;
+
+	private final List<String> configuredURLs;
+
 	private volatile Client client;
 
-	private int  mRestClientConnTimeOutMs;
-	private int  mRestClientReadTimeOutMs;
 
-	public RangerRESTClient() {
-		this(RangerConfiguration.getInstance().get(RANGER_PROP_POLICYMGR_URL),
-			 RangerConfiguration.getInstance().get(RANGER_PROP_POLICYMGR_SSLCONFIG_FILENAME));
-	}
-
-	public RangerRESTClient(String url, String sslConfigFileName) {
+	public RangerRESTClient(String url, String sslConfigFileName, Configuration config) {
 		mUrl               = url;
 		mSslConfigFileName = sslConfigFileName;
+		configuredURLs     = StringUtil.getURLs(mUrl);
 
-		init();
+		setLastKnownActiveUrlIndex((new Random()).nextInt(getConfiguredURLs().size()));
+
+		init(config);
 	}
 
 	public String getUrl() {
@@ -211,7 +218,7 @@ public class RangerRESTClient {
 			client = Client.create(config);
 		}
 
-		if(!StringUtils.isEmpty(mUsername) && !StringUtils.isEmpty(mPassword)) {
+		if(StringUtils.isNotEmpty(mUsername) && StringUtils.isNotEmpty(mPassword)) {
 			client.addFilter(new HTTPBasicAuthFilter(mUsername, mPassword));
 		}
 
@@ -226,37 +233,35 @@ public class RangerRESTClient {
 		client = null;
 	}
 
-	private void init() {
+	private void init(Configuration config) {
 		try {
 			gsonBuilder = new GsonBuilder().setDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ").create();
 		} catch(Throwable excp) {
 			LOG.fatal("RangerRESTClient.init(): failed to create GsonBuilder object", excp);
 		}
 
-		mIsSSL = StringUtil.containsIgnoreCase(mUrl, "https");
+		mIsSSL = StringUtils.containsIgnoreCase(mUrl, "https");
 
 		if (mIsSSL) {
 
 			InputStream in = null;
 
 			try {
-				Configuration conf = new Configuration();
-
 				in = getFileInputStream(mSslConfigFileName);
 
 				if (in != null) {
-					conf.addResource(in);
+					config.addResource(in);
 				}
 
-				mKeyStoreURL = conf.get(RANGER_POLICYMGR_CLIENT_KEY_FILE_CREDENTIAL);
+				mKeyStoreURL = config.get(RANGER_POLICYMGR_CLIENT_KEY_FILE_CREDENTIAL);
 				mKeyStoreAlias = RANGER_POLICYMGR_CLIENT_KEY_FILE_CREDENTIAL_ALIAS;
-				mKeyStoreType = conf.get(RANGER_POLICYMGR_CLIENT_KEY_FILE_TYPE, RANGER_POLICYMGR_CLIENT_KEY_FILE_TYPE_DEFAULT);
-				mKeyStoreFile = conf.get(RANGER_POLICYMGR_CLIENT_KEY_FILE);
+				mKeyStoreType = config.get(RANGER_POLICYMGR_CLIENT_KEY_FILE_TYPE, RANGER_POLICYMGR_CLIENT_KEY_FILE_TYPE_DEFAULT);
+				mKeyStoreFile = config.get(RANGER_POLICYMGR_CLIENT_KEY_FILE);
 
-				mTrustStoreURL = conf.get(RANGER_POLICYMGR_TRUSTSTORE_FILE_CREDENTIAL);
+				mTrustStoreURL = config.get(RANGER_POLICYMGR_TRUSTSTORE_FILE_CREDENTIAL);
 				mTrustStoreAlias = RANGER_POLICYMGR_TRUSTSTORE_FILE_CREDENTIAL_ALIAS;
-				mTrustStoreType = conf.get(RANGER_POLICYMGR_TRUSTSTORE_FILE_TYPE, RANGER_POLICYMGR_TRUSTSTORE_FILE_TYPE_DEFAULT);
-				mTrustStoreFile = conf.get(RANGER_POLICYMGR_TRUSTSTORE_FILE);
+				mTrustStoreType = config.get(RANGER_POLICYMGR_TRUSTSTORE_FILE_TYPE, RANGER_POLICYMGR_TRUSTSTORE_FILE_TYPE_DEFAULT);
+				mTrustStoreFile = config.get(RANGER_POLICYMGR_TRUSTSTORE_FILE);
 			} catch (IOException ioe) {
 				LOG.error("Unable to load SSL Config FileName: [" + mSslConfigFileName + "]", ioe);
 			} finally {
@@ -271,25 +276,32 @@ public class RangerRESTClient {
 
 		String keyStoreFilepwd = getCredential(mKeyStoreURL, mKeyStoreAlias);
 
-		if (!StringUtil.isEmpty(mKeyStoreFile) && !StringUtil.isEmpty(keyStoreFilepwd)) {
+		kmList = getKeyManagers(mKeyStoreFile,keyStoreFilepwd);
+		return kmList;
+	}
+
+	public KeyManager[] getKeyManagers(String keyStoreFile, String keyStoreFilePwd) {
+		KeyManager[] kmList = null;
+
+		if (StringUtils.isNotEmpty(keyStoreFile) && StringUtils.isNotEmpty(keyStoreFilePwd)) {
 			InputStream in =  null;
 
 			try {
-				in = getFileInputStream(mKeyStoreFile);
+				in = getFileInputStream(keyStoreFile);
 
 				if (in != null) {
 					KeyStore keyStore = KeyStore.getInstance(mKeyStoreType);
 
-					keyStore.load(in, keyStoreFilepwd.toCharArray());
+					keyStore.load(in, keyStoreFilePwd.toCharArray());
 
 					KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance(RANGER_SSL_KEYMANAGER_ALGO_TYPE);
 
-					keyManagerFactory.init(keyStore, keyStoreFilepwd.toCharArray());
+					keyManagerFactory.init(keyStore, keyStoreFilePwd.toCharArray());
 
 					kmList = keyManagerFactory.getKeyManagers();
 				} else {
-					LOG.error("Unable to obtain keystore from file [" + mKeyStoreFile + "]");
-					throw new IllegalStateException("Unable to find keystore file :" + mKeyStoreFile);
+					LOG.error("Unable to obtain keystore from file [" + keyStoreFile + "]");
+					throw new IllegalStateException("Unable to find keystore file :" + keyStoreFile);
 				}
 			} catch (KeyStoreException e) {
 				LOG.error("Unable to obtain from KeyStore :" + e.getMessage(), e);
@@ -302,15 +314,15 @@ public class RangerRESTClient {
 				throw new IllegalStateException("Unable to obtain the requested certification :" + e.getMessage(), e);
 			} catch (FileNotFoundException e) {
 				LOG.error("Unable to find the necessary SSL Keystore Files", e);
-				throw new IllegalStateException("Unable to find keystore file :" + mKeyStoreFile + ", error :" + e.getMessage(), e);
+				throw new IllegalStateException("Unable to find keystore file :" + keyStoreFile + ", error :" + e.getMessage(), e);
 			} catch (IOException e) {
 				LOG.error("Unable to read the necessary SSL Keystore Files", e);
-				throw new IllegalStateException("Unable to read keystore file :" + mKeyStoreFile + ", error :" + e.getMessage(), e);
+				throw new IllegalStateException("Unable to read keystore file :" + keyStoreFile + ", error :" + e.getMessage(), e);
 			} catch (UnrecoverableKeyException e) {
 				LOG.error("Unable to recover the key from keystore", e);
-				throw new IllegalStateException("Unable to recover the key from keystore :" + mKeyStoreFile+", error :" + e.getMessage(), e);
+				throw new IllegalStateException("Unable to recover the key from keystore :" + keyStoreFile+", error :" + e.getMessage(), e);
 			} finally {
-				close(in, mKeyStoreFile);
+				close(in, keyStoreFile);
 			}
 		}
 
@@ -319,14 +331,23 @@ public class RangerRESTClient {
 
 	private TrustManager[] getTrustManagers() {
 		TrustManager[] tmList = null;
+		if (StringUtils.isNotEmpty(mTrustStoreURL) && StringUtils.isNotEmpty(mTrustStoreAlias)) {
+			String trustStoreFilepwd = getCredential(mTrustStoreURL, mTrustStoreAlias);
+			if (StringUtils.isNotEmpty(trustStoreFilepwd)) {
+				tmList = getTrustManagers(mTrustStoreFile, trustStoreFilepwd);
+			}
+		}
+		return tmList;
+	}
 
-		String trustStoreFilepwd = getCredential(mTrustStoreURL, mTrustStoreAlias);
+	public TrustManager[] getTrustManagers(String trustStoreFile, String trustStoreFilepwd) {
+		TrustManager[] tmList = null;
 
-		if (!StringUtil.isEmpty(mTrustStoreFile) && !StringUtil.isEmpty(trustStoreFilepwd)) {
+		if (StringUtils.isNotEmpty(trustStoreFile) && StringUtils.isNotEmpty(trustStoreFilepwd)) {
 			InputStream in =  null;
 
 			try {
-				in = getFileInputStream(mTrustStoreFile);
+				in = getFileInputStream(trustStoreFile);
 
 				if (in != null) {
 					KeyStore trustStore = KeyStore.getInstance(mTrustStoreType);
@@ -339,8 +360,8 @@ public class RangerRESTClient {
 
 					tmList = trustManagerFactory.getTrustManagers();
 				} else {
-					LOG.error("Unable to obtain truststore from file [" + mTrustStoreFile + "]");
-					throw new IllegalStateException("Unable to find truststore file :" + mTrustStoreFile);
+					LOG.error("Unable to obtain truststore from file [" + trustStoreFile + "]");
+					throw new IllegalStateException("Unable to find truststore file :" + trustStoreFile);
 				}
 			} catch (KeyStoreException e) {
 				LOG.error("Unable to obtain from KeyStore", e);
@@ -352,21 +373,33 @@ public class RangerRESTClient {
 				LOG.error("Unable to obtain the requested certification :" + e.getMessage(), e);
 				throw new IllegalStateException("Unable to obtain the requested certification :" + e.getMessage(), e);
 			} catch (FileNotFoundException e) {
-				LOG.error("Unable to find the necessary SSL TrustStore File:" + mTrustStoreFile, e);
-				throw new IllegalStateException("Unable to find trust store file :" + mTrustStoreFile + ", error :" + e.getMessage(), e);
+				LOG.error("Unable to find the necessary SSL TrustStore File:" + trustStoreFile, e);
+				throw new IllegalStateException("Unable to find trust store file :" + trustStoreFile + ", error :" + e.getMessage(), e);
 			} catch (IOException e) {
-				LOG.error("Unable to read the necessary SSL TrustStore Files :" + mTrustStoreFile, e);
-				throw new IllegalStateException("Unable to read the trust store file :" + mTrustStoreFile + ", error :" + e.getMessage(), e);
+				LOG.error("Unable to read the necessary SSL TrustStore Files :" + trustStoreFile, e);
+				throw new IllegalStateException("Unable to read the trust store file :" + trustStoreFile + ", error :" + e.getMessage(), e);
 			} finally {
-				close(in, mTrustStoreFile);
+				close(in, trustStoreFile);
 			}
 		}
 		
 		return tmList;
 	}
 
-	private SSLContext getSSLContext(KeyManager[] kmList, TrustManager[] tmList) {
-	        Validate.notNull(tmList, "TrustManager is not specified");
+	protected SSLContext getSSLContext(KeyManager[] kmList, TrustManager[] tmList) {
+		if (tmList == null) {
+			try {
+			 String algo = TrustManagerFactory.getDefaultAlgorithm() ;
+			 TrustManagerFactory tmf =  TrustManagerFactory.getInstance(algo) ;
+			 tmf.init((KeyStore)null) ;
+			 tmList = tmf.getTrustManagers() ;
+			}
+			catch(NoSuchAlgorithmException | KeyStoreException | IllegalStateException  e) {
+				LOG.error("Unable to get the default SSL TrustStore for the JVM",e);
+				tmList = null;
+			}
+		}
+	    Validate.notNull(tmList, "TrustManager is not specified");
 		try {
 			SSLContext sslContext = SSLContext.getInstance(RANGER_SSL_CONTEXT_ALGO_TYPE);
 
@@ -389,7 +422,7 @@ public class RangerRESTClient {
 	private InputStream getFileInputStream(String fileName)  throws IOException {
 		InputStream in = null;
 
-		if(! StringUtil.isEmpty(fileName)) {
+		if(StringUtils.isNotEmpty(fileName)) {
 			File f = new File(fileName);
 
 			if (f.exists()) {
@@ -412,4 +445,184 @@ public class RangerRESTClient {
 			}
 		}
 	}
+
+	public ClientResponse get(String relativeUrl, Map<String, String> params) throws Exception {
+		ClientResponse finalResponse = null;
+		int startIndex = this.lastKnownActiveUrlIndex;
+		int currentIndex = 0;
+
+		for (int index = 0; index < configuredURLs.size(); index++) {
+			try {
+				currentIndex = (startIndex + index) % configuredURLs.size();
+
+				WebResource webResource = getClient().resource(configuredURLs.get(currentIndex) + relativeUrl);
+				webResource = setQueryParams(webResource, params);
+
+				finalResponse = webResource.accept(RangerRESTUtils.REST_EXPECTED_MIME_TYPE).type(RangerRESTUtils.REST_MIME_TYPE_JSON).get(ClientResponse.class);
+
+				if (finalResponse != null) {
+					setLastKnownActiveUrlIndex(currentIndex);
+					break;
+				}
+			} catch (ClientHandlerException ex) {
+				LOG.warn("Failed to communicate with Ranger Admin, URL : " + configuredURLs.get(currentIndex));
+				processException(index, ex);
+			}
+		}
+		return finalResponse;
+	}
+
+	public ClientResponse post(String relativeUrl, Map<String, String> params, Object obj) throws Exception {
+		ClientResponse finalResponse = null;
+		int startIndex = this.lastKnownActiveUrlIndex;
+		int currentIndex = 0;
+
+		for (int index = 0; index < configuredURLs.size(); index++) {
+			try {
+				currentIndex = (startIndex + index) % configuredURLs.size();
+
+				WebResource webResource = getClient().resource(configuredURLs.get(currentIndex) + relativeUrl);
+				webResource = setQueryParams(webResource, params);
+				finalResponse = webResource.accept(RangerRESTUtils.REST_EXPECTED_MIME_TYPE).type(RangerRESTUtils.REST_MIME_TYPE_JSON).post(ClientResponse.class, toJson(obj));
+				if (finalResponse != null) {
+					setLastKnownActiveUrlIndex(currentIndex);
+					break;
+				}
+			} catch (ClientHandlerException ex) {
+				LOG.warn("Failed to communicate with Ranger Admin, URL : " + configuredURLs.get(currentIndex));
+				processException(index, ex);
+			}
+		}
+		return finalResponse;
+	}
+
+	public ClientResponse delete(String relativeUrl, Map<String, String> params) throws Exception {
+		ClientResponse finalResponse = null;
+		int startIndex = this.lastKnownActiveUrlIndex;
+		int currentIndex = 0;
+
+		for (int index = 0; index < configuredURLs.size(); index++) {
+			try {
+				currentIndex = (startIndex + index) % configuredURLs.size();
+
+				WebResource webResource = getClient().resource(configuredURLs.get(currentIndex) + relativeUrl);
+				webResource = setQueryParams(webResource, params);
+
+				finalResponse = webResource.accept(RangerRESTUtils.REST_EXPECTED_MIME_TYPE).type(RangerRESTUtils.REST_MIME_TYPE_JSON).delete(ClientResponse.class);
+				if (finalResponse != null) {
+					setLastKnownActiveUrlIndex(currentIndex);
+					break;
+				}
+			} catch (ClientHandlerException ex) {
+				LOG.warn("Failed to communicate with Ranger Admin, URL : " + configuredURLs.get(currentIndex));
+				processException(index, ex);
+			}
+		}
+		return finalResponse;
+	}
+
+	public ClientResponse put(String relativeUrl, Map<String, String> params, Object obj) throws Exception {
+		ClientResponse finalResponse = null;
+		int startIndex = this.lastKnownActiveUrlIndex;
+		int currentIndex = 0;
+		for (int index = 0; index < configuredURLs.size(); index++) {
+			try {
+				currentIndex = (startIndex + index) % configuredURLs.size();
+
+				WebResource webResource = getClient().resource(configuredURLs.get(currentIndex) + relativeUrl);
+				webResource = setQueryParams(webResource, params);
+				finalResponse = webResource.accept(RangerRESTUtils.REST_EXPECTED_MIME_TYPE).type(RangerRESTUtils.REST_MIME_TYPE_JSON).put(ClientResponse.class, toJson(obj));
+				if (finalResponse != null) {
+					setLastKnownActiveUrlIndex(currentIndex);
+					break;
+				}
+			} catch (ClientHandlerException ex) {
+				LOG.warn("Failed to communicate with Ranger Admin, URL : " + configuredURLs.get(currentIndex));
+				processException(index, ex);
+			}
+		}
+		return finalResponse;
+	}
+
+	public ClientResponse put(String relativeURL, Object request, Cookie sessionId) throws Exception {
+		ClientResponse response = null;
+		int startIndex = this.lastKnownActiveUrlIndex;
+		int currentIndex = 0;
+
+		for (int index = 0; index < configuredURLs.size(); index++) {
+			try {
+				currentIndex = (startIndex + index) % configuredURLs.size();
+
+				WebResource webResource = createWebResourceForCookieAuth(currentIndex, relativeURL);
+				WebResource.Builder br = webResource.getRequestBuilder().cookie(sessionId);
+				response = br.accept(RangerRESTUtils.REST_EXPECTED_MIME_TYPE).type(RangerRESTUtils.REST_MIME_TYPE_JSON)
+						.put(ClientResponse.class, toJson(request));
+				if (response != null) {
+					setLastKnownActiveUrlIndex(currentIndex);
+					break;
+				}
+			} catch (ClientHandlerException e) {
+				LOG.warn("Failed to communicate with Ranger Admin, URL : " + configuredURLs.get(currentIndex));
+				processException(index, e);
+			}
+		}
+		return response;
+	}
+
+	protected static WebResource setQueryParams(WebResource webResource, Map<String, String> params) {
+		WebResource ret = webResource;
+		if (webResource != null && params != null) {
+			Set<Map.Entry<String, String>> entrySet= params.entrySet();
+			for (Map.Entry<String, String> entry : entrySet) {
+				ret = ret.queryParam(entry.getKey(), entry.getValue());
+			}
+		}
+		return ret;
+	}
+
+	protected void setLastKnownActiveUrlIndex(int lastKnownActiveUrlIndex) {
+		this.lastKnownActiveUrlIndex = lastKnownActiveUrlIndex;
+	}
+
+	protected WebResource createWebResourceForCookieAuth(int currentIndex, String relativeURL) {
+		Client cookieClient = getClient();
+		cookieClient.removeAllFilters();
+		WebResource ret = cookieClient.resource(configuredURLs.get(currentIndex) + relativeURL);
+		return ret;
+	}
+
+	protected void processException(int index, ClientHandlerException e) throws Exception {
+		if (index == configuredURLs.size() - 1) {
+			LOG.error("Failed to communicate with all Ranger Admin's URL's : [ " + configuredURLs + " ]");
+			throw e;
+		}
+	}
+
+	public int getLastKnownActiveUrlIndex() {
+		return lastKnownActiveUrlIndex;
+	}
+
+	public List<String> getConfiguredURLs() {
+		return configuredURLs;
+	}
+
+	public boolean isSSL() {
+		return mIsSSL;
+	}
+
+	public void setSSL(boolean mIsSSL) {
+		this.mIsSSL = mIsSSL;
+	}
+
+	protected void setClient(Client client) {
+		this.client = client;
+	}
+
+	protected void setKeyStoreType(String mKeyStoreType) {
+		this.mKeyStoreType = mKeyStoreType;
+	}
+
+	protected void setTrustStoreType(String mTrustStoreType) {
+		this.mTrustStoreType = mTrustStoreType;
+	}
 }
