@@ -23,8 +23,11 @@ import java.util.*;
 import java.util.Map.Entry;
 import java.security.PrivilegedExceptionAction;
 
+import com.google.protobuf.Message;
 import com.google.protobuf.Service;
 import org.apache.commons.collections.CollectionUtils;
+import org.apache.commons.collections.MapUtils;
+import org.apache.commons.lang.StringUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
@@ -55,7 +58,6 @@ import org.apache.hadoop.hbase.util.Pair;
 import org.apache.hadoop.hbase.wal.WALEdit;
 import org.apache.hadoop.security.AccessControlException;
 import org.apache.ranger.audit.model.AuthzAuditEvent;
-import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
 import org.apache.ranger.authorization.hadoop.constants.RangerHadoopConstants;
 import org.apache.ranger.authorization.utils.StringUtil;
 import org.apache.ranger.plugin.audit.RangerDefaultAuditHandler;
@@ -86,6 +88,8 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 	private UserProvider userProvider;
     private RegionCoprocessorEnvironment regionEnv;
 	private Map<InternalScanner, String> scannerOwners = new MapMaker().weakKeys().makeMap();
+	/** if we should check EXEC permissions */
+	private boolean shouldCheckExecPermission;
 	
 	/*
 	 * These are package level only for testability and aren't meant to be exposed outside via getters/setters or made available to derived classes.
@@ -307,7 +311,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 	}
 	
 	ColumnFamilyAccessResult evaluateAccess(ObserverContext<?> ctx, String operation, Action action, final RegionCoprocessorEnvironment env,
-											final Map<byte[], ? extends Collection<?>> familyMap) throws AccessDeniedException {
+											final Map<byte[], ? extends Collection<?>> familyMap, String commandStr) throws AccessDeniedException {
 
 		String access = _authUtils.getAccess(action);
 		User user = getActiveUser(ctx);
@@ -343,6 +347,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		HbaseAuditHandler auditHandler = _factory.getAuditHandler();
 		AuthorizationSession session = new AuthorizationSession(hbasePlugin)
 				.operation(operation)
+				.otherInformation(commandStr)
 				.remoteAddress(getRemoteAddress())
 				.auditHandler(auditHandler)
 				.user(user)
@@ -523,7 +528,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		return result;
 	}
 
-	Filter authorizeAccess(ObserverContext<?> ctx, String operation, Action action, final RegionCoprocessorEnvironment env, final Map<byte[], NavigableSet<byte[]>> familyMap) throws AccessDeniedException {
+	Filter authorizeAccess(ObserverContext<?> ctx, String operation, Action action, final RegionCoprocessorEnvironment env, final Map<byte[], NavigableSet<byte[]>> familyMap, String commandStr) throws AccessDeniedException {
 
 		if (LOG.isDebugEnabled()) {
 			LOG.debug("==> authorizeAccess");
@@ -533,8 +538,8 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		try {
 			perf = RangerPerfTracer.getPerfTracer(PERF_HBASEAUTH_REQUEST_LOG, "RangerAuthorizationCoprocessor.authorizeAccess(request=Operation[" + operation + "]");
 
-			ColumnFamilyAccessResult accessResult = evaluateAccess(ctx, operation, action, env, familyMap);
-			RangerDefaultAuditHandler auditHandler = new RangerDefaultAuditHandler();
+			ColumnFamilyAccessResult accessResult = evaluateAccess(ctx, operation, action, env, familyMap, commandStr);
+			RangerDefaultAuditHandler auditHandler = new RangerDefaultAuditHandler(hbasePlugin.getConfig());
 			if (accessResult._everythingIsAccessible) {
 				auditHandler.logAuthzAudits(accessResult._accessAllowedEvents);
 				auditHandler.logAuthzAudits(accessResult._familyLevelAccessEvents);
@@ -576,8 +581,8 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 			if (RangerPerfTracer.isPerfTraceEnabled(PERF_HBASEAUTH_REQUEST_LOG)) {
 				perf = RangerPerfTracer.getPerfTracer(PERF_HBASEAUTH_REQUEST_LOG, "RangerAuthorizationCoprocessor.requirePermission(request=Operation[" + operation + "]");
 			}
-			ColumnFamilyAccessResult accessResult = evaluateAccess(ctx, operation, action, regionServerEnv, familyMap);
-			RangerDefaultAuditHandler auditHandler = new RangerDefaultAuditHandler();
+			ColumnFamilyAccessResult accessResult = evaluateAccess(ctx, operation, action, regionServerEnv, familyMap, null);
+			RangerDefaultAuditHandler auditHandler = new RangerDefaultAuditHandler(hbasePlugin.getConfig());
 			if (accessResult._everythingIsAccessible) {
 				auditHandler.logAuthzAudits(accessResult._accessAllowedEvents);
 				auditHandler.logAuthzAudits(accessResult._familyLevelAccessEvents);
@@ -986,13 +991,16 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		if (LOG.isDebugEnabled()) {
 			LOG.debug("==> preScannerOpen");
 		}
-
+		String commandStr = null;
 		try {
 			RegionCoprocessorEnvironment e = c.getEnvironment();
 
 			Map<byte[], NavigableSet<byte[]>> familyMap = scan.getFamilyMap();
-			String operation = "scannerOpen";
-			Filter filter = authorizeAccess(c, operation, Action.READ, e, familyMap);
+			String operation    = "scannerOpen";
+			byte[] tableName    = getTableName(e);
+			String tableNameStr = tableName != null ?  new String(tableName):" ";
+			commandStr          = getCommandString(HbaseConstants.SCAN, tableNameStr, scan.toMap());
+			Filter filter       = authorizeAccess(c, operation, Action.READ, e, familyMap, commandStr);
 			if (filter == null) {
 				if (LOG.isDebugEnabled()) {
 					LOG.debug("preScannerOpen: Access allowed for all families/column.  No filter added");
@@ -1007,7 +1015,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 			}
 		} finally {
 			if (LOG.isDebugEnabled()) {
-				LOG.debug("<== preScannerOpen");
+				LOG.debug("<== preScannerOpen: commandStr: " + commandStr);
 			}
 		}
 	}
@@ -1072,6 +1080,9 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 	public void start(CoprocessorEnvironment env) throws IOException {
 		String appType = "unknown";
 
+		shouldCheckExecPermission = env.getConfiguration().getBoolean(
+				AccessControlConstants.EXEC_PERMISSION_CHECKS_KEY,
+				AccessControlConstants.DEFAULT_EXEC_PERMISSION_CHECKS);
 		if (env instanceof MasterCoprocessorEnvironment) {
 			coprocessorType = MASTER_COPROCESSOR_TYPE;
 			appType = "hbaseMaster";
@@ -1101,7 +1112,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 
 					plugin.init();
 
-					UpdateRangerPoliciesOnGrantRevoke = RangerConfiguration.getInstance().getBoolean(RangerHadoopConstants.HBASE_UPDATE_RANGER_POLICIES_ON_GRANT_REVOKE_PROP, RangerHadoopConstants.HBASE_UPDATE_RANGER_POLICIES_ON_GRANT_REVOKE_DEFAULT_VALUE);
+					UpdateRangerPoliciesOnGrantRevoke = plugin.getConfig().getBoolean(RangerHadoopConstants.HBASE_UPDATE_RANGER_POLICIES_ON_GRANT_REVOKE_PROP, RangerHadoopConstants.HBASE_UPDATE_RANGER_POLICIES_ON_GRANT_REVOKE_DEFAULT_VALUE);
 
 					hbasePlugin = plugin;
 				}
@@ -1123,12 +1134,16 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		if (LOG.isDebugEnabled()) {
 			LOG.debug("==> preGetOp");
 		}
+		String commandStr = null;
 		try {
 			RegionCoprocessorEnvironment e = rEnv.getEnvironment();
 			Map<byte[], NavigableSet<byte[]>> familyMap = get.getFamilyMap();
 
-			String operation = "get";
-			Filter filter = authorizeAccess(rEnv, operation, Action.READ, e, familyMap);
+			String operation    = "get";
+			byte[] tableName    = getTableName(e);
+			String tableNameStr = tableName != null ?  new String(tableName):" ";
+			commandStr          = getCommandString(HbaseConstants.GET, tableNameStr, get.toMap());
+			Filter filter       = authorizeAccess(rEnv, operation, Action.READ, e, familyMap, commandStr);
 			if (filter == null) {
 				if (LOG.isDebugEnabled()) {
 					LOG.debug("preGetOp: all access allowed, no filter returned");
@@ -1143,7 +1158,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 			}
 		} finally {
 			if (LOG.isDebugEnabled()) {
-				LOG.debug("<== preGetOp");
+				LOG.debug("<== preGetOp: commandStr: " + commandStr );
 			}
 		}
 	}
@@ -1222,7 +1237,23 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 
 		requirePermission(ctx, "preCleanupBulkLoad", Permission.Action.WRITE, ctx.getEnvironment(), cfs);
 	}
-	
+
+	/* ---- EndpointObserver implementation ---- */
+
+	@Override
+	public Message preEndpointInvocation(ObserverContext<RegionCoprocessorEnvironment> ctx,
+										 Service service, String methodName, Message request) throws IOException {
+		// Don't intercept calls to our own AccessControlService, we check for
+		// appropriate permissions in the service handlers
+		if (shouldCheckExecPermission && !(service instanceof AccessControlService)) {
+			requirePermission(ctx,
+					"invoke(" + service.getDescriptorForType().getName() + "." + methodName + ")",
+					getTableName(ctx.getEnvironment()), null, null,
+					Action.EXEC);
+		}
+		return request;
+	}
+
 	@Override
 	public void grant(RpcController controller, AccessControlProtos.GrantRequest request, RpcCallback<AccessControlProtos.GrantResponse> done) {
 		boolean isSuccess = false;
@@ -1237,7 +1268,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 
 				if(plugin != null) {
 
-					RangerAccessResultProcessor auditHandler = new RangerDefaultAuditHandler();
+					RangerAccessResultProcessor auditHandler = new RangerDefaultAuditHandler(hbasePlugin.getConfig());
 
 					plugin.grantAccess(grData, auditHandler);
 
@@ -1277,7 +1308,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 
 				if(plugin != null) {
 
-					RangerAccessResultProcessor auditHandler = new RangerDefaultAuditHandler();
+					RangerAccessResultProcessor auditHandler = new RangerDefaultAuditHandler(hbasePlugin.getConfig());
 
 					plugin.revokeAccess(grData, auditHandler);
 
@@ -1324,7 +1355,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 				}
 			}
 			RangerAccessRequestImpl rangerAccessrequest = new RangerAccessRequestImpl(resource, null,
-					_userUtils.getUserAsString(user), groups);
+					_userUtils.getUserAsString(user), groups, null);
 			rangerAccessrequest.setAction(operation);
 			rangerAccessrequest.setClientIPAddress(getRemoteAddress());
 			rangerAccessrequest.setResourceMatchingScope(RangerAccessRequest.ResourceMatchingScope.SELF);
@@ -1336,8 +1367,8 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 				perms = User.runAsLoginUser(new PrivilegedExceptionAction<List<UserPermission>>() {
 					@Override
 					public List<UserPermission> run() throws Exception {
-						return getUserPrermissions(
-								hbasePlugin.getCurrentRangerAuthContext().getResourceACLs(rangerAccessrequest),
+						return getUserPermissions(
+								hbasePlugin.getResourceACLs(rangerAccessrequest),
 								table.getNameAsString(), false);
 					}
 				});
@@ -1349,8 +1380,8 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 				perms = User.runAsLoginUser(new PrivilegedExceptionAction<List<UserPermission>>() {
 					@Override
 					public List<UserPermission> run() throws Exception {
-						return getUserPrermissions(
-								hbasePlugin.getCurrentRangerAuthContext().getResourceACLs(rangerAccessrequest),
+						return getUserPermissions(
+								hbasePlugin.getResourceACLs(rangerAccessrequest),
 								namespace, true);
 					}
 				});
@@ -1359,8 +1390,8 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 				perms = User.runAsLoginUser(new PrivilegedExceptionAction<List<UserPermission>>() {
 					@Override
 					public List<UserPermission> run() throws Exception {
-						return getUserPrermissions(
-								hbasePlugin.getCurrentRangerAuthContext().getResourceACLs(rangerAccessrequest), null,
+						return getUserPermissions(
+								hbasePlugin.getResourceACLs(rangerAccessrequest), null,
 								false);
 					}
 				});
@@ -1377,8 +1408,8 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		done.run(response);
 	}
 
-	private List<UserPermission> getUserPrermissions(RangerResourceACLs rangerResourceACLs, String resource,
-			boolean isNamespace) {
+	private List<UserPermission> getUserPermissions(RangerResourceACLs rangerResourceACLs, String resource,
+                                                    boolean isNamespace) {
 		List<UserPermission> userPermissions = new ArrayList<UserPermission>();
 		Action[] hbaseActions = Action.values();
 		List<String> hbaseActionsList = new ArrayList<String>();
@@ -1398,7 +1429,7 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 			String user = !isGroup ? userAcls.getKey() : AuthUtil.toGroupEntry(userAcls.getKey());
 			List<Action> allowedPermissions = new ArrayList<Action>();
 			for (Entry<String, AccessResult> permissionAccess : userAcls.getValue().entrySet()) {
-				String permission = permissionAccess.getKey().toUpperCase();
+				String permission = _authUtils.getActionName(permissionAccess.getKey());
 				if (hbaseActionsList.contains(permission)
 						&& permissionAccess.getValue().getResult() == RangerPolicyEvaluator.ACCESS_ALLOWED) {
 					allowedPermissions.add(Action.valueOf(permission));
@@ -1495,6 +1526,9 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		ret.setReplaceExistingPermissions(Boolean.TRUE);
 		ret.setResource(mapResource);
 		ret.setClientIPAddress(getRemoteAddress());
+		ret.setForwardedAddresses(null);//TODO: Need to check with Knox proxy how they handle forwarded add.
+		ret.setRemoteIPAddress(getRemoteAddress());
+		ret.setRequestData(up.toString());
 
 		if(userName.startsWith(GROUP_PREFIX)) {
 			ret.getGroups().add(userName.substring(GROUP_PREFIX.length()));
@@ -1520,7 +1554,9 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 					ret.getAccessTypes().add(HbaseAuthUtils.ACCESS_TYPE_ADMIN);
 					ret.setDelegateAdmin(Boolean.TRUE);
 				break;
-
+				case 'X':
+					ret.getAccessTypes().add(HbaseAuthUtils.ACCESS_TYPE_EXECUTE);
+				break;
 				default:
 					LOG.warn("grant(): ignoring action '" + action.name() + "' for user '" + userName + "'");
 			}
@@ -1600,7 +1636,10 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		ret.setReplaceExistingPermissions(Boolean.TRUE);
 		ret.setResource(mapResource);
 		ret.setClientIPAddress(getRemoteAddress());
-
+		ret.setForwardedAddresses(null);//TODO: Need to check with Knox proxy how they handle forwarded add.
+		ret.setRemoteIPAddress(getRemoteAddress());
+		ret.setRequestData(up.toString());
+		
 		if(userName.startsWith(GROUP_PREFIX)) {
 			ret.getGroups().add(userName.substring(GROUP_PREFIX.length()));
 		} else {
@@ -1612,9 +1651,112 @@ public class RangerAuthorizationCoprocessor implements AccessControlService.Inte
 		ret.getAccessTypes().add(HbaseAuthUtils.ACCESS_TYPE_WRITE);
 		ret.getAccessTypes().add(HbaseAuthUtils.ACCESS_TYPE_CREATE);
 		ret.getAccessTypes().add(HbaseAuthUtils.ACCESS_TYPE_ADMIN);
+		ret.getAccessTypes().add(HbaseAuthUtils.ACCESS_TYPE_EXECUTE);
 
 		return ret;
 	}
+	private String getCommandString(String operationName, String tableNameStr, Map<String,Object> opMetaData) {
+		StringBuilder ret = new StringBuilder();
+		if (!HbaseConstants.HBASE_META_TABLE.equals(tableNameStr)) {
+			ret.append(operationName);
+			ret.append(HbaseConstants.SPACE);
+			ret.append(tableNameStr).append(HbaseConstants.COMMA).append(HbaseConstants.SPACE);
+			ret.append(getPredicates(operationName, opMetaData));
+		}
+		return ret.toString();
+	}
+
+	private String getPredicates(String operationName, Map<String,Object> opMetaData) {
+		StringBuilder ret = new StringBuilder();
+
+		if (MapUtils.isNotEmpty(opMetaData)) {
+			HashMap<String, ArrayList<?>> families    = (HashMap<String, ArrayList<?>>) opMetaData.get(HbaseConstants.FAMILIES);
+			String 						  startRowVal = (String) opMetaData.get(HbaseConstants.STARTROW);
+			String                        stopRowVal  = (String) opMetaData.get(HbaseConstants.STOPROW);
+			String                        filterVal   = (String) opMetaData.get(HbaseConstants.FILTER);
+			String                        rowVal      = (String) opMetaData.get(HbaseConstants.ROW);
+
+			if (!isQueryforInfo(families)) {
+				ret.append(HbaseConstants.OPEN_BRACES);
+				if (HbaseConstants.SCAN.equals(operationName)) {
+					if (StringUtils.isNotEmpty(startRowVal)) {
+						ret.append(formatPredicate(ret, PredicateType.STARTROW, startRowVal));
+					}
+					if (StringUtils.isNotEmpty(stopRowVal)) {
+						ret.append(formatPredicate(ret, PredicateType.STOPROW, stopRowVal));
+					}
+				} else {
+					if(StringUtils.isNotEmpty(rowVal)) {
+						ret.append(formatPredicate(ret, PredicateType.ROW, rowVal));
+					}
+				}
+				if (StringUtils.isNotEmpty(filterVal)) {
+					ret.append(formatPredicate(ret, PredicateType.FILTER, filterVal));
+				}
+				if (MapUtils.isNotEmpty(families)) {
+					String colfamily = families.toString();
+					ret.append(formatPredicate(ret, PredicateType.COLUMNS, colfamily));
+				}
+				ret.append(HbaseConstants.SPACE).append(HbaseConstants.CLOSED_BRACES);
+			}
+		}
+		return ret.toString();
+	}
+
+	private boolean isQueryforInfo(HashMap<String, ArrayList<?>> families) {
+		boolean ret = false;
+		for(HashMap.Entry family : families.entrySet()) {
+			String familyKey = (String) family.getKey();
+			if (HbaseConstants.INFO.equals(familyKey)) {
+				ret = true;
+				break;
+			}
+		}
+		return ret;
+	}
+
+	private String formatPredicate(StringBuilder commandStr, PredicateType predicateType, String val) {
+		StringBuilder ret = new StringBuilder();
+		if (HbaseConstants.OPEN_BRACES.equals(commandStr.toString())) {
+			ret.append(HbaseConstants.SPACE);
+		} else {
+			ret.append(HbaseConstants.COMMA).append(HbaseConstants.SPACE);
+		}
+		ret.append(buildPredicate(predicateType, val));
+		return ret.toString();
+	}
+
+	private String buildPredicate(PredicateType predicateType, String val) {
+		StringBuilder ret = new StringBuilder();
+		switch (predicateType) {
+			case STARTROW:
+				ret.append(PredicateType.STARTROW.name().toUpperCase());
+				ret.append(HbaseConstants.ARROW);
+				ret.append(HbaseConstants.SINGLE_QUOTES).append(val).append(HbaseConstants.SINGLE_QUOTES);
+				break;
+			case STOPROW:
+				ret.append(PredicateType.STOPROW.name().toUpperCase());
+				ret.append(HbaseConstants.ARROW);
+				ret.append(HbaseConstants.SINGLE_QUOTES).append(val).append(HbaseConstants.SINGLE_QUOTES);
+				break;
+			case FILTER:
+				ret.append(PredicateType.FILTER.name().toUpperCase());
+				ret.append(HbaseConstants.ARROW);
+				ret.append(HbaseConstants.SINGLE_QUOTES).append(val).append(HbaseConstants.SINGLE_QUOTES);
+				break;
+			case COLUMNS:
+				ret.append(PredicateType.COLUMNS.name().toUpperCase());
+				ret.append(HbaseConstants.ARROW);
+				ret.append(HbaseConstants.SINGLE_QUOTES).append(val).append(HbaseConstants.SINGLE_QUOTES);
+				break;
+			case ROW:
+				ret.append(val);
+				break;
+		}
+		return ret.toString();
+	}
+
+	enum PredicateType {STARTROW, STOPROW, FILTER, COLUMNS, ROW};
 }
 
 
