@@ -26,17 +26,14 @@ import static org.apache.ranger.authorization.hadoop.constants.RangerHadoopConst
 
 import java.net.InetAddress;
 import java.security.SecureRandom;
-import java.util.Date;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Map;
-import java.util.Set;
-import java.util.Stack;
+import java.util.*;
 
 import org.apache.commons.lang.ArrayUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.permission.FsAction;
 import org.apache.hadoop.hdfs.DFSUtil;
 import org.apache.hadoop.hdfs.server.namenode.INode;
@@ -49,7 +46,7 @@ import org.apache.hadoop.ipc.Server;
 import org.apache.hadoop.security.AccessControlException;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.ranger.audit.model.AuthzAuditEvent;
-import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
+import org.apache.ranger.authorization.hadoop.config.RangerPluginConfig;
 import org.apache.ranger.authorization.hadoop.constants.RangerHadoopConstants;
 import org.apache.ranger.authorization.hadoop.exceptions.RangerAccessControlException;
 import org.apache.ranger.plugin.audit.RangerDefaultAuditHandler;
@@ -80,12 +77,19 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 
 	private RangerHdfsPlugin           rangerPlugin            = null;
 	private Map<FsAction, Set<String>> access2ActionListMapper = new HashMap<FsAction, Set<String>>();
+	private final Path                 addlConfigFile;
 
 	public RangerHdfsAuthorizer() {
+		this(null);
+	}
+
+	public RangerHdfsAuthorizer(Path addlConfigFile) {
 		if(LOG.isDebugEnabled()) {
 			LOG.debug("==> RangerHdfsAuthorizer.RangerHdfsAuthorizer()");
 		}
 
+		this.addlConfigFile = addlConfigFile;
+
 		if(LOG.isDebugEnabled()) {
 			LOG.debug("<== RangerHdfsAuthorizer.RangerHdfsAuthorizer()");
 		}
@@ -96,10 +100,11 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 			LOG.debug("==> RangerHdfsAuthorizer.start()");
 		}
 
-		RangerHdfsPlugin plugin = new RangerHdfsPlugin();
+		RangerHdfsPlugin plugin = new RangerHdfsPlugin(addlConfigFile);
+
 		plugin.init();
 
-		if (RangerHdfsPlugin.isOptimizeSubAccessAuthEnabled()) {
+		if (plugin.isOptimizeSubAccessAuthEnabled()) {
 			LOG.info(RangerHadoopConstants.RANGER_OPTIMIZE_SUBACCESS_AUTHORIZATION_PROP + " is enabled");
 		}
 
@@ -181,6 +186,10 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 		return rangerAce;
 	}
 
+	// for testing
+	public Configuration getConfig() {
+		return rangerPlugin.getConfig();
+	}
 
 	private enum AuthzStatus { ALLOW, DENY, NOT_DETERMINED };
 
@@ -311,7 +320,7 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 					parent   = inodes.length > 1 ? inodes[inodes.length - 2] : null;
 					inode    = inodes[inodes.length - 1]; // could be null while creating a new file
 
-					auditHandler = doNotGenerateAuditRecord ? null : new RangerHdfsAuditHandler(resourcePath, isTraverseOnlyCheck);
+					auditHandler = doNotGenerateAuditRecord ? null : new RangerHdfsAuditHandler(resourcePath, isTraverseOnlyCheck, rangerPlugin.getHadoopModuleName(), rangerPlugin.getExcludedUsers());
 
 					/* Hadoop versions prior to 2.8.0 didn't ask for authorization of parent/ancestor traversal for
 					 * reading or writing a file. However, Hadoop version 2.8.0 and later ask traversal authorization for
@@ -400,7 +409,7 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 
 								AuthzStatus subDirAuthStatus = AuthzStatus.NOT_DETERMINED;
 
-								boolean optimizeSubAccessAuthEnabled = RangerHdfsPlugin.isOptimizeSubAccessAuthEnabled();
+								boolean optimizeSubAccessAuthEnabled = rangerPlugin.isOptimizeSubAccessAuthEnabled();
 
 								if (optimizeSubAccessAuthEnabled) {
 									subDirAuthStatus = isAccessAllowedForHierarchy(data.dir, dirAttribs, data.resourcePath, subAccess, user, groups, plugin);
@@ -409,7 +418,7 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 								if (subDirAuthStatus != AuthzStatus.ALLOW) {
 									for(INode child : cList) {
 										if (child.isDirectory()) {
-											directories.push(new SubAccessData(child.asDirectory(), resourcePath + org.apache.hadoop.fs.Path.SEPARATOR_CHAR + child.getLocalName()));
+											directories.push(new SubAccessData(child.asDirectory(), resourcePath + Path.SEPARATOR_CHAR + child.getLocalName()));
 										}
 									}
 								}
@@ -576,7 +585,7 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 			}
 
 			AuthzStatus authzStatus = AuthzStatus.NOT_DETERMINED;
-			if(RangerHdfsPlugin.isHadoopAuthEnabled() && defaultEnforcer != null) {
+			if(rangerPlugin.isHadoopAuthEnabled() && defaultEnforcer != null) {
 
 				RangerPerfTracer hadoopAuthPerf = null;
 
@@ -715,10 +724,10 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 				}
 
 				String subDirPath = path;
-				if (subDirPath.charAt(subDirPath.length() - 1) != org.apache.hadoop.fs.Path.SEPARATOR_CHAR) {
-					subDirPath = subDirPath + Character.toString(org.apache.hadoop.fs.Path.SEPARATOR_CHAR);
+				if (subDirPath.charAt(subDirPath.length() - 1) != Path.SEPARATOR_CHAR) {
+					subDirPath = subDirPath + Character.toString(Path.SEPARATOR_CHAR);
 				}
-				subDirPath = subDirPath + RangerHdfsPlugin.getRandomizedWildcardPathName();
+				subDirPath = subDirPath + rangerPlugin.getRandomizedWildcardPathName();
 
 				for (String accessType : accessTypes) {
 					RangerHdfsAccessRequest request = new RangerHdfsAccessRequest(null, subDirPath, pathOwner, access, accessType, user, groups);
@@ -754,24 +763,47 @@ public class RangerHdfsAuthorizer extends INodeAttributeProvider {
 
 
 class RangerHdfsPlugin extends RangerBasePlugin {
-	private static boolean hadoopAuthEnabled = RangerHadoopConstants.RANGER_ADD_HDFS_PERMISSION_DEFAULT;
-	private static String fileNameExtensionSeparator;
-	private static boolean optimizeSubAccessAuthEnabled = RangerHadoopConstants.RANGER_OPTIMIZE_SUBACCESS_AUTHORIZATION_DEFAULT;
-	private static String randomizedWildcardPathName;
+	private static final Log LOG = LogFactory.getLog(RangerHdfsPlugin.class);
 
-	public RangerHdfsPlugin() {
+	private static String fileNameExtensionSeparator = RangerHdfsAuthorizer.DEFAULT_FILENAME_EXTENSION_SEPARATOR;
+
+	private final boolean     hadoopAuthEnabled;
+	private final boolean     optimizeSubAccessAuthEnabled;
+	private final String      randomizedWildcardPathName;
+	private final String      hadoopModuleName;
+	private final Set<String> excludeUsers = new HashSet<>();
+
+	public RangerHdfsPlugin(Path addlConfigFile) {
 		super("hdfs", "hdfs");
-	}
 
-	public void init() {
-		super.init();
+		RangerPluginConfig config = getConfig();
 
-		RangerHdfsPlugin.hadoopAuthEnabled = RangerConfiguration.getInstance().getBoolean(RangerHadoopConstants.RANGER_ADD_HDFS_PERMISSION_PROP, RangerHadoopConstants.RANGER_ADD_HDFS_PERMISSION_DEFAULT);
-		RangerHdfsPlugin.fileNameExtensionSeparator = RangerConfiguration.getInstance().get(RangerHdfsAuthorizer.RANGER_FILENAME_EXTENSION_SEPARATOR_PROP, RangerHdfsAuthorizer.DEFAULT_FILENAME_EXTENSION_SEPARATOR);
-		RangerHdfsPlugin.optimizeSubAccessAuthEnabled = RangerConfiguration.getInstance().getBoolean(RangerHadoopConstants.RANGER_OPTIMIZE_SUBACCESS_AUTHORIZATION_PROP, RangerHadoopConstants.RANGER_OPTIMIZE_SUBACCESS_AUTHORIZATION_DEFAULT);
+		if (addlConfigFile != null) {
+			config.addResource(addlConfigFile);
+		}
 
 		String random = generateString("^&#@!%()-_+=@:;'<>`~abcdefghijklmnopqrstuvwxyz01234567890");
-		randomizedWildcardPathName = RangerPathResourceMatcher.WILDCARD_ASTERISK + random + RangerPathResourceMatcher.WILDCARD_ASTERISK;
+
+		RangerHdfsPlugin.fileNameExtensionSeparator = config.get(RangerHdfsAuthorizer.RANGER_FILENAME_EXTENSION_SEPARATOR_PROP, RangerHdfsAuthorizer.DEFAULT_FILENAME_EXTENSION_SEPARATOR);
+
+		this.hadoopAuthEnabled            = config.getBoolean(RangerHadoopConstants.RANGER_ADD_HDFS_PERMISSION_PROP, RangerHadoopConstants.RANGER_ADD_HDFS_PERMISSION_DEFAULT);
+		this.optimizeSubAccessAuthEnabled = config.getBoolean(RangerHadoopConstants.RANGER_OPTIMIZE_SUBACCESS_AUTHORIZATION_PROP, RangerHadoopConstants.RANGER_OPTIMIZE_SUBACCESS_AUTHORIZATION_DEFAULT);
+		this.randomizedWildcardPathName   = RangerPathResourceMatcher.WILDCARD_ASTERISK + random + RangerPathResourceMatcher.WILDCARD_ASTERISK;
+		this.hadoopModuleName             = config.get(RangerHadoopConstants.AUDITLOG_HADOOP_MODULE_ACL_NAME_PROP , RangerHadoopConstants.DEFAULT_HADOOP_MODULE_ACL_NAME);
+
+		String excludeUserList = config.get(RangerHadoopConstants.AUDITLOG_HDFS_EXCLUDE_LIST_PROP, RangerHadoopConstants.AUDITLOG_EMPTY_STRING);
+
+		if (excludeUserList != null && excludeUserList.trim().length() > 0) {
+			for(String excludeUser : excludeUserList.trim().split(",")) {
+				excludeUser = excludeUser.trim();
+
+				if (LOG.isDebugEnabled()) {
+					LOG.debug("Adding exclude user [" + excludeUser + "]");
+				}
+
+				excludeUsers.add(excludeUser);
+			}
+		}
 	}
 
 	// Build random string of length between 56 and 112 characters
@@ -794,18 +826,21 @@ class RangerHdfsPlugin extends RangerBasePlugin {
 		return new String(text);
 	}
 
-	public static boolean isHadoopAuthEnabled() {
-		return RangerHdfsPlugin.hadoopAuthEnabled;
-	}
 	public static String getFileNameExtensionSeparator() {
-		return RangerHdfsPlugin.fileNameExtensionSeparator;
+		return fileNameExtensionSeparator;
 	}
-	public static boolean isOptimizeSubAccessAuthEnabled() {
-		return RangerHdfsPlugin.optimizeSubAccessAuthEnabled;
+
+	public boolean isHadoopAuthEnabled() {
+		return hadoopAuthEnabled;
 	}
-	public static String getRandomizedWildcardPathName() {
-		return RangerHdfsPlugin.randomizedWildcardPathName;
+	public boolean isOptimizeSubAccessAuthEnabled() {
+		return optimizeSubAccessAuthEnabled;
 	}
+	public String getRandomizedWildcardPathName() {
+		return randomizedWildcardPathName;
+	}
+	public String getHadoopModuleName() { return hadoopModuleName; }
+	public Set<String> getExcludedUsers() { return  excludeUsers; }
 }
 
 class RangerHdfsResource extends RangerAccessResourceImpl {
@@ -826,6 +861,8 @@ class RangerHdfsAccessRequest extends RangerAccessRequestImpl {
 		super.setAccessTime(new Date());
 		super.setClientIPAddress(getRemoteIp());
 		super.setAction(access.toString());
+		super.setForwardedAddresses(null);
+		super.setRemoteIPAddress(getRemoteIp());
 
 		if (inode != null) {
 			buildRequestContext(inode);
@@ -861,26 +898,14 @@ class RangerHdfsAuditHandler extends RangerDefaultAuditHandler {
 	private final String pathToBeValidated;
 	private final boolean auditOnlyIfDenied;
 
-	private static final String    HadoopModuleName = RangerConfiguration.getInstance().get(RangerHadoopConstants.AUDITLOG_HADOOP_MODULE_ACL_NAME_PROP , RangerHadoopConstants.DEFAULT_HADOOP_MODULE_ACL_NAME);
-	private static final String    excludeUserList  = RangerConfiguration.getInstance().get(RangerHadoopConstants.AUDITLOG_HDFS_EXCLUDE_LIST_PROP, RangerHadoopConstants.AUDITLOG_EMPTY_STRING);
-	private static HashSet<String> excludeUsers     = null;
+	private final String      hadoopModuleName;
+	private final Set<String> excludeUsers;
 
-	static {
-		if (excludeUserList != null && excludeUserList.trim().length() > 0) {
-			excludeUsers = new HashSet<String>();
-			for(String excludeUser : excludeUserList.trim().split(",")) {
-				excludeUser = excludeUser.trim();
-				if (LOG.isDebugEnabled()) {
-					LOG.debug("Adding exclude user [" + excludeUser + "]");
-				}
-				excludeUsers.add(excludeUser);
-				}
-		}
-	}
-
-	public RangerHdfsAuditHandler(String pathToBeValidated, boolean auditOnlyIfDenied) {
+	public RangerHdfsAuditHandler(String pathToBeValidated, boolean auditOnlyIfDenied, String hadoopModuleName, Set<String> excludedUsers) {
 		this.pathToBeValidated = pathToBeValidated;
 		this.auditOnlyIfDenied = auditOnlyIfDenied;
+		this.hadoopModuleName  = hadoopModuleName;
+		this.excludeUsers      = excludedUsers;
 	}
 
 	@Override
@@ -889,32 +914,34 @@ class RangerHdfsAuditHandler extends RangerDefaultAuditHandler {
 			LOG.debug("==> RangerHdfsAuditHandler.logAudit(" + result + ")");
 		}
 
-		if(! isAuditEnabled && result.getIsAudited()) {
-			isAuditEnabled = true;
-		}
+		if (result != null) {
+			if(! isAuditEnabled && result.getIsAudited()) {
+				isAuditEnabled = true;
+			}
 
-		if (auditEvent == null) {
-			auditEvent = super.getAuthzEvents(result);
-		}
+			if (auditEvent == null) {
+				auditEvent = super.getAuthzEvents(result);
+			}
 
-		if (auditEvent != null) {
-			RangerAccessRequest request = result.getAccessRequest();
-			RangerAccessResource resource = request.getResource();
-			String resourcePath = resource != null ? resource.getAsString() : null;
+			if (auditEvent != null) {
+				RangerAccessRequest request = result.getAccessRequest();
+				RangerAccessResource resource = request.getResource();
+				String resourcePath = resource != null ? resource.getAsString() : null;
 
-			// Overwrite fields in original auditEvent
-			auditEvent.setEventTime(request.getAccessTime() != null ? request.getAccessTime() : new Date());
-			auditEvent.setAccessType(request.getAction());
-			auditEvent.setResourcePath(this.pathToBeValidated);
-			auditEvent.setResultReason(resourcePath);
+				// Overwrite fields in original auditEvent
+				auditEvent.setEventTime(request.getAccessTime() != null ? request.getAccessTime() : new Date());
+				auditEvent.setAccessType(request.getAction());
+				auditEvent.setResourcePath(this.pathToBeValidated);
+				auditEvent.setResultReason(resourcePath);
 
-			auditEvent.setAccessResult((short) (result.getIsAllowed() ? 1 : 0));
-			auditEvent.setPolicyId(result.getPolicyId());
-			auditEvent.setPolicyVersion(result.getPolicyVersion());
+				auditEvent.setAccessResult((short) (result.getIsAllowed() ? 1 : 0));
+				auditEvent.setPolicyId(result.getPolicyId());
+				auditEvent.setPolicyVersion(result.getPolicyVersion());
 
-			Set<String> tags = getTags(request);
-			if (tags != null) {
-				auditEvent.setTags(tags);
+				Set<String> tags = getTags(request);
+				if (tags != null) {
+					auditEvent.setTags(tags);
+				}
 			}
 		}
 
@@ -932,7 +959,7 @@ class RangerHdfsAuditHandler extends RangerDefaultAuditHandler {
 			auditEvent.setResultReason(path);
 			auditEvent.setAccessResult((short) (accessGranted ? 1 : 0));
 			auditEvent.setAccessType(action == null ? null : action.toString());
-			auditEvent.setAclEnforcer(HadoopModuleName);
+			auditEvent.setAclEnforcer(hadoopModuleName);
 			auditEvent.setPolicyId(-1);
 		}
 
