@@ -139,13 +139,12 @@ public class EagerKeyGeneratorKeyProviderCryptoExtension
 	@Override
 	public EncryptedKeyVersion reencryptEncryptedKey(EncryptedKeyVersion arg0)
 			throws IOException, GeneralSecurityException {
-		// TODO Auto-generated method stub
-		return null;
+      return keyProviderCryptoExtension.reencryptEncryptedKey(arg0);
 	}
 
 	@Override
 	public void reencryptEncryptedKeys(List<EncryptedKeyVersion> arg0) throws IOException, GeneralSecurityException {
-		// TODO Auto-generated method stub
+      keyProviderCryptoExtension.reencryptEncryptedKeys(arg0);
 	}
 
   }
@@ -165,6 +164,16 @@ public class EagerKeyGeneratorKeyProviderCryptoExtension
         new CryptoExtension(conf, keyProviderCryptoExtension));
   }
 
+  /**
+     * Roll a new version of the given key generating the material for it.
+     * <p>
+     * Due to the caching on the ValueQueue, even after a rollNewVersion call,
+     * {@link #generateEncryptedKey(String)} may still return an old key - even
+     * when we drain the queue here, the async thread may later fill in old keys.
+     * This is acceptable since old version keys are still able to decrypt, and
+     * client shall make no assumptions that it will get a new versioned key
+     * after rollNewVersion.
+  */
   @Override
   public KeyVersion rollNewVersion(String name)
       throws NoSuchAlgorithmException, IOException {
@@ -180,4 +189,11 @@ public class EagerKeyGeneratorKeyProviderCryptoExtension
     getExtension().drain(name);
     return keyVersion;
   }
+
+  @Override
+  public void invalidateCache(String name) throws IOException {
+    super.invalidateCache(name);
+    getExtension().drain(name);
+    }
+
 }
