@@ -55,7 +55,7 @@ def check_output(query):
 	elif os_name == "WINDOWS":
 		p = subprocess.Popen(query, stdout=subprocess.PIPE, shell=True)
 	output = p.communicate ()[0]
-	return output
+	return output.decode()
 
 def log(msg,type):
 	if type == 'info':
@@ -423,7 +423,7 @@ class BaseDB(object):
 						key3 = int(version.strip("J"))
 						my_dict[key3] = filename
 
-			keylist = my_dict.keys()
+			keylist = list(my_dict)
 			keylist.sort()
 			for key in keylist:
 				#print "%s: %s" % (key, my_dict[key])
@@ -808,7 +808,19 @@ class OracleConf(BaseDB):
 		return ret
 
 	def get_check_table_query(self, TABLE_NAME):
+		CT=self.commandTerminator
+		get_cmd = self.get_jisql_cmd(self.db_user, self.db_password, self.db_name)
+		if is_unix:
+			query = get_cmd + CT + " -query 'select default_tablespace from user_users;'"
+		elif os_name == "WINDOWS":
+			query = get_cmd + " -query \"select default_tablespace from user_users;\" -c ;"
+		jisql_log(query, self.db_password)
+		output = check_output(query).strip()
+		output = output.strip(' |')
 		db_name=self.db_name.upper()
+		if (db_name =='' and output is not None and output != ''):
+			db_name = output
+		#db_name could be given db_name or user's default tablespace name
 		return "select UPPER(table_name) from all_tables where UPPER(tablespace_name)=UPPER('%s') and UPPER(table_name)=UPPER('%s');" %(db_name ,TABLE_NAME)
 
 	def get_unstale_patch_query(self, version, isActive, client_host,stalePatchEntryHoldTimeInMinutes):
@@ -1035,20 +1047,20 @@ def main(argv):
 		lib_home = os.path.join(RANGER_ADMIN_HOME,"ews","webapp","WEB-INF","lib","*")
 		get_ranger_version_cmd="%s -cp %s org.apache.ranger.common.RangerVersionInfo"%(JAVA_BIN,lib_home)
 		ranger_version = check_output(get_ranger_version_cmd).split("\n")[1]
-	except Exception, error:
+	except Exception as error:
 		ranger_version=''
 
 	try:
 		if ranger_version=="" or ranger_version=="ranger-admin - None":
 			script_path = os.path.join(RANGER_ADMIN_HOME,"ews","ranger-admin-services.sh")
 			ranger_version=check_output(script_path +" version").split("\n")[1]
-	except Exception, error:
+	except Exception as error:
 		ranger_version=''
 
 	try:
 		if ranger_version=="" or ranger_version=="ranger-admin - None":
 			ranger_version=check_output("ranger-admin version").split("\n")[1]
-	except Exception, error:
+	except Exception as error:
 		ranger_version=''
 
 	if ranger_version=="" or ranger_version is None:
