@@ -72,6 +72,12 @@ javax_net_ssl_keyStorePassword=$(get_prop 'javax_net_ssl_keyStorePassword' $PROP
 javax_net_ssl_trustStore=$(get_prop 'javax_net_ssl_trustStore' $PROPFILE)
 javax_net_ssl_trustStorePassword=$(get_prop 'javax_net_ssl_trustStorePassword' $PROPFILE)
 audit_store=$(get_prop 'audit_store' $PROPFILE)
+audit_elasticsearch_urls=$(get_prop 'audit_elasticsearch_urls' $PROPFILE)
+audit_elasticsearch_port=$(get_prop 'audit_elasticsearch_port' $PROPFILE)
+audit_elasticsearch_user=$(get_prop 'audit_elasticsearch_user' $PROPFILE)
+audit_elasticsearch_password=$(get_prop 'audit_elasticsearch_password' $PROPFILE)
+audit_elasticsearch_index=$(get_prop 'audit_elasticsearch_index' $PROPFILE)
+audit_elasticsearch_bootstrap_enabled=$(get_prop 'audit_elasticsearch_bootstrap_enabled' $PROPFILE)
 audit_solr_urls=$(get_prop 'audit_solr_urls' $PROPFILE)
 audit_solr_user=$(get_prop 'audit_solr_user' $PROPFILE)
 audit_solr_password=$(get_prop 'audit_solr_password' $PROPFILE)
@@ -150,7 +156,7 @@ audit_solr_no_shards=$(get_prop 'audit_solr_no_shards' $PROPFILE)
 audit_solr_no_replica=$(get_prop 'audit_solr_no_replica' $PROPFILE)
 audit_solr_max_shards_per_node=$(get_prop 'audit_solr_max_shards_per_node' $PROPFILE)
 audit_solr_acl_user_list_sasl=$(get_prop 'audit_solr_acl_user_list_sasl' $PROPFILE)
-
+audit_solr_bootstrap_enabled=$(get_prop 'audit_solr_bootstrap_enabled' $PROPFILE)
 
 DB_HOST="${db_host}"
 
@@ -221,7 +227,7 @@ getPropertyFromFile(){
 #Update Properties to File
 #$1 -> propertyName $2 -> newPropertyValue $3 -> fileName
 updatePropertyToFilePy(){
-    python update_property.py $1 "${2}" $3
+    $PYTHON_COMMAND_INVOKER update_property.py $1 "${2}" $3
         check_ret_status $? "Update property failed for: " $1
 }
 
@@ -247,6 +253,16 @@ init_variables(){
 			exit 1
 		fi
 	fi
+	if [ "${audit_store}" == "elasticsearch" ] ;then
+		if [ "${audit_elasticsearch_urls}" == "" ] ;then
+			log "[I] Please provide valid URL for 'elasticsearch' audit store!"
+			exit 1
+		fi
+		if [ "${audit_elasticsearch_port}" == "" ] ;then
+			log "[I] Please provide valid port for 'elasticsearch' audit store!"
+			exit 1
+		fi
+	fi
 
 	db_ssl_enabled=`echo $db_ssl_enabled | tr '[:upper:]' '[:lower:]'`
 	if [ "${db_ssl_enabled}" != "true" ]
@@ -291,7 +307,7 @@ run_dba_steps(){
 		log "[I] Setup mode is set to SeparateDBA. Not Running DBA steps. Please run dba_script.py before running setup..!";
 	else
 		log "[I] Setup mode is not set. Running DBA steps..";
-                python dba_script.py -q
+                $PYTHON_COMMAND_INVOKER dba_script.py -q
         fi
 }
 check_ranger_version(){
@@ -697,6 +713,38 @@ update_properties() {
 		propertyName=ranger.audit.solr.urls
 		newPropertyValue=${audit_solr_urls}
 		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger
+
+		propertyName=ranger.audit.solr.bootstrap.enabled
+		newPropertyValue=${audit_solr_bootstrap_enabled}
+		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger
+	fi
+
+	if [ "${audit_store}" == "elasticsearch" ]
+	then
+		propertyName=ranger.audit.elasticsearch.urls
+		newPropertyValue=${audit_elasticsearch_urls}
+		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger
+
+		propertyName=ranger.audit.elasticsearch.port
+		newPropertyValue=${audit_elasticsearch_port}
+		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger
+
+		propertyName=ranger.audit.elasticsearch.user
+		newPropertyValue=${audit_elasticsearch_user}
+		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger
+
+		propertyName=ranger.audit.elasticsearch.password
+		newPropertyValue=${audit_elasticsearch_password}
+		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger
+
+		propertyName=ranger.audit.elasticsearch.index
+		newPropertyValue=${audit_elasticsearch_index}
+		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger
+
+		propertyName=ranger.audit.elasticsearch.bootstrap.enabled
+		newPropertyValue=${audit_elasticsearch_bootstrap_enabled}
+		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger
+
 	fi
 
 	if [ "${audit_store}" != "" ]
