@@ -29,13 +29,15 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-
 import javax.servlet.http.HttpServletResponse;
+
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.io.FilenameUtils;
 import org.apache.commons.io.IOCase;
 import org.apache.commons.lang.StringUtils;
+import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.log4j.Logger;
+import org.apache.ranger.authorization.hadoop.config.RangerAdminConfig;
 import org.apache.ranger.common.AppConstants;
 import org.apache.ranger.common.ContextUtil;
 import org.apache.ranger.common.GUIDUtil;
@@ -46,6 +48,7 @@ import org.apache.ranger.common.RangerConstants;
 import org.apache.ranger.common.StringUtil;
 import org.apache.ranger.common.UserSessionBase;
 import org.apache.ranger.db.RangerDaoManager;
+import org.apache.ranger.db.XXDBBaseDao;
 import org.apache.ranger.entity.XXAsset;
 import org.apache.ranger.entity.XXDBBase;
 import org.apache.ranger.entity.XXGroup;
@@ -59,6 +62,10 @@ import org.apache.ranger.entity.XXUser;
 import org.apache.ranger.plugin.model.RangerBaseModelObject;
 import org.apache.ranger.plugin.model.RangerService;
 import org.apache.ranger.plugin.store.EmbeddedServiceDefsUtil;
+import org.apache.ranger.rest.ServiceREST;
+import org.apache.ranger.security.context.RangerAdminOpContext;
+import org.apache.ranger.security.context.RangerContextHolder;
+import org.apache.ranger.service.XUserService;
 import org.apache.ranger.view.VXPortalUser;
 import org.apache.ranger.view.VXResource;
 import org.apache.ranger.view.VXResponse;
@@ -84,6 +91,9 @@ public class RangerBizUtil {
 	@Autowired
 	UserMgr userMgr;
 
+	@Autowired
+	XUserService xUserService;
+
 	@Autowired
 	GUIDUtil guidUtil;
 	
@@ -100,12 +110,21 @@ public class RangerBizUtil {
 	private static int PATH_CHAR_SET_LEN = PATH_CHAR_SET.length;
 	public static final String AUDIT_STORE_RDBMS = "DB";
 	public static final String AUDIT_STORE_SOLR = "solr";
+	public static final String AUDIT_STORE_ElasticSearch = "elasticSearch";
+	public static final boolean batchClearEnabled = PropertiesUtil.getBooleanProperty("ranger.jpa.jdbc.batch-clear.enable", true);
+	public static final int policyBatchSize = PropertiesUtil.getIntProperty("ranger.jpa.jdbc.batch-clear.size", 10);
+	public static final int batchPersistSize = PropertiesUtil.getIntProperty("ranger.jpa.jdbc.batch-persist.size", 500);
 
 	String auditDBType = AUDIT_STORE_RDBMS;
+	private final boolean allowUnauthenticatedAccessInSecureEnvironment;
 
 	static String fileSeparator = PropertiesUtil.getProperty("ranger.file.separator", "/");
 
 	public RangerBizUtil() {
+		RangerAdminConfig config = RangerAdminConfig.getInstance();
+
+		allowUnauthenticatedAccessInSecureEnvironment = config.getBoolean("ranger.admin.allow.unauthenticated.access", false);
+
 		maxFirstNameLength = Integer.parseInt(PropertiesUtil.getProperty("ranger.user.firstname.maxlength", "16"));
 		maxDisplayNameLength = PropertiesUtil.getIntProperty("ranger.bookmark.name.maxlen", maxDisplayNameLength);
 
@@ -436,6 +455,17 @@ public class RangerBizUtil {
 		return matchFound;
 	}
 
+	public void failUnauthenticatedIfNotAllowed() throws Exception {
+		if (UserGroupInformation.isSecurityEnabled()) {
+			UserSessionBase currentUserSession = ContextUtil.getCurrentUserSession();
+			if (currentUserSession == null) {
+				if (!allowUnauthenticatedAccessInSecureEnvironment) {
+					throw new Exception("Unauthenticated access not allowed");
+				}
+			}
+		}
+	}
+
 	/**
 	 * returns true if user is having required permission on given Hbase
 	 * resource
@@ -876,10 +906,8 @@ public class RangerBizUtil {
 	private boolean checkUsrPermForPolicy(Long xUserId, int permission,
 			Long resourceId) {
 		// this snippet load user groups and permission map list from DB
-		List<XXGroup> userGroups = new ArrayList<XXGroup>();
-		List<XXPermMap> permMapList = new ArrayList<XXPermMap>();
-		userGroups = daoManager.getXXGroup().findByUserId(xUserId);
-		permMapList = daoManager.getXXPermMap().findByResourceId(resourceId);
+		List<XXGroup> userGroups = daoManager.getXXGroup().findByUserId(xUserId);
+		List<XXPermMap> permMapList = daoManager.getXXPermMap().findByResourceId(resourceId);
 		Long publicGroupId = getPublicGroupId();
 		boolean matchFound = false;
 		for (XXPermMap permMap : permMapList) {
@@ -887,9 +915,8 @@ public class RangerBizUtil {
 				if (permMap.getPermFor() == AppConstants.XA_PERM_FOR_GROUP) {
 					// check whether permission is enabled for public group or a
 					// group to which user belongs
-					matchFound = (publicGroupId != null && publicGroupId == permMap
-							.getGroupId())
-							|| isGroupInList(permMap.getGroupId(), userGroups);
+					matchFound = (publicGroupId != null && publicGroupId.equals(permMap.getGroupId())) ||
+											 isGroupInList(permMap.getGroupId(), userGroups);
 				} else if (permMap.getPermFor() == AppConstants.XA_PERM_FOR_USER) {
 					// check whether permission is enabled to user
 					matchFound = permMap.getUserId().equals(xUserId);
@@ -1148,46 +1175,6 @@ public class RangerBizUtil {
 		}
 	}
 
-	public void createTrxLog(List<XXTrxLog> trxLogList, boolean flush) {
-		if (trxLogList == null) {
-			return;
-		}
-
-		UserSessionBase usb = ContextUtil.getCurrentUserSession();
-		Long authSessionId = null;
-		if (usb != null) {
-			authSessionId = ContextUtil.getCurrentUserSession().getSessionId();
-		}
-		if(guidUtil != null){
-		Long trxId = guidUtil.genLong();
-		for (XXTrxLog xTrxLog : trxLogList) {
-			if (xTrxLog != null) {
-				if ("Password".equalsIgnoreCase(StringUtil.trim(xTrxLog.getAttributeName()))) {
-					if (xTrxLog.getPreviousValue() != null
-							&& !xTrxLog.getPreviousValue().trim().isEmpty()
-							&& !"null".equalsIgnoreCase(xTrxLog
-									.getPreviousValue().trim())) {
-						xTrxLog.setPreviousValue(AppConstants.Masked_String);
-					}
-					if (xTrxLog.getNewValue() != null
-							&& !xTrxLog.getNewValue().trim().isEmpty()
-							&& !"null".equalsIgnoreCase(xTrxLog.getNewValue()
-									.trim())) {
-						xTrxLog.setNewValue(AppConstants.Masked_String);
-					}
-				}
-				xTrxLog.setTransactionId(trxId.toString());
-				if (authSessionId != null) {
-					xTrxLog.setSessionId("" + authSessionId);
-				}
-				xTrxLog.setSessionType("Spring Authenticated Session");
-				xTrxLog.setRequestId(trxId.toString());
-				daoManager.getXXTrxLog().create(xTrxLog, flush);
-			}
-		}
-		}
-	}
-
 	public static int getDBFlavor() {
 		String[] propertyNames = { "xa.db.flavor",
 									"ranger.jpa.jdbc.dialect",
@@ -1425,12 +1412,31 @@ public class RangerBizUtil {
 		return false;
 	}
 
-	public boolean isUserAllowedForGrantRevoke(RangerService rangerService,
-			String cfgNameAllowedUsers, String userName) {
+	public boolean isUserAllowedForGrantRevoke(RangerService rangerService, String userName) {
+		return isUserInConfigParameter(rangerService, ServiceREST.Allowed_User_List_For_Grant_Revoke, userName);
+	}
+
+	public boolean isUserRangerAdmin(String username) {
+		boolean isAdmin = false;
+		try {
+			VXUser vxUser = xUserService.getXUserByUserName(username);
+			if (vxUser != null && (vxUser.getUserRoleList().contains(RangerConstants.ROLE_ADMIN) || vxUser.getUserRoleList().contains(RangerConstants.ROLE_SYS_ADMIN))) {
+				isAdmin = true;
+			}
+		} catch (Exception ex) {
+		}
+		return isAdmin;
+	}
+
+	public boolean isUserServiceAdmin(RangerService rangerService, String userName) {
+		return isUserInConfigParameter(rangerService, ServiceDBStore.SERVICE_ADMIN_USERS, userName);
+	}
+
+	public boolean isUserInConfigParameter(RangerService rangerService, String configParamName, String userName) {
 		Map<String, String> map = rangerService.getConfigs();
 
-		if (map != null && map.containsKey(cfgNameAllowedUsers)) {
-			String userNames = map.get(cfgNameAllowedUsers);
+		if (map != null && map.containsKey(configParamName)) {
+			String userNames = map.get(configParamName);
 			String[] userList = userNames.split(",");
 			if (userList != null) {
 				for (String u : userList) {
@@ -1441,7 +1447,7 @@ public class RangerBizUtil {
 			}
 		}
 		return false;
-	}	
+	}
 
         public void blockAuditorRoleUser() {
                 UserSessionBase session = ContextUtil.getCurrentUserSession();
@@ -1499,4 +1505,44 @@ public class RangerBizUtil {
 			}
 		}
 	}
+
+	public static boolean isBulkMode() {
+		return ContextUtil.isBulkModeContext();
+	}
+
+	public static boolean setBulkMode(boolean val) {
+		if(RangerContextHolder.getOpContext()!=null){
+			RangerContextHolder.getOpContext().setBulkModeContext(val);
+		}
+		else {
+			  RangerAdminOpContext opContext = new RangerAdminOpContext();
+			  opContext.setBulkModeContext(val);
+			  RangerContextHolder.setOpContext(opContext);
+		}
+		return isBulkMode();
+	}
+
+	//should be used only in bulk operation like importPolicies, policies delete.
+	public void bulkModeOnlyFlushAndClear() {
+		if (batchClearEnabled) {
+			XXDBBaseDao xXDBBaseDao = daoManager.getXXDBBase();
+			if (xXDBBaseDao != null) {
+				xXDBBaseDao.flush();
+				xXDBBaseDao.clear();
+			}
+		}
+	}
+
+	public boolean checkAdminAccess() {
+		UserSessionBase currentUserSession = ContextUtil.getCurrentUserSession();
+		if (currentUserSession != null) {
+			return currentUserSession.isUserAdmin();
+		} else {
+			VXResponse vXResponse = new VXResponse();
+			vXResponse.setStatusCode(HttpServletResponse.SC_UNAUTHORIZED);
+			vXResponse.setMsgDesc("Bad Credentials");
+			throw restErrorUtil.generateRESTException(vXResponse);
+		}
+	}
+
 }
