@@ -39,6 +39,7 @@ import org.apache.ranger.entity.XXPolicyLabelMap;
 import org.apache.ranger.entity.XXPortalUser;
 import org.apache.ranger.entity.XXSecurityZone;
 import org.apache.ranger.entity.XXService;
+import org.apache.ranger.entity.XXServiceDef;
 import org.apache.ranger.plugin.model.RangerPolicy;
 import org.apache.ranger.plugin.model.RangerPolicy.RangerDataMaskPolicyItem;
 import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyItem;
@@ -472,6 +473,7 @@ public class RangerPolicyRetriever {
 		final XXService              service;
 		final ListIterator<XXPolicy> iterPolicy;
 		final ListIterator<XXPolicyLabelMap> iterPolicyLabels;
+		final XXServiceDef           serviceDef;
 
 		RetrieverContext(XXService xService) {
 			if (xService != null) {
@@ -486,10 +488,12 @@ public class RangerPolicyRetriever {
 				lookupCache.setConditionNameMapping(daoMgr.getXXPolicyRefCondition().findUpdatedConditionNamesByService(serviceId));
 
 				this.service    = xService;
+				this.serviceDef = daoMgr.getXXServiceDef().getById(xService.getType());
 				this.iterPolicy = daoMgr.getXXPolicy().findByServiceId(serviceId).listIterator();
 				this.iterPolicyLabels = daoMgr.getXXPolicyLabelMap().findByServiceId(serviceId).listIterator();
 			} else {
 				this.service    = null;
+				this.serviceDef = null;
 				this.iterPolicy = null;
 				this.iterPolicyLabels = null;
 			}
@@ -507,8 +511,10 @@ public class RangerPolicyRetriever {
 			lookupCache.setConditionNameMapping(daoMgr.getXXPolicyRefCondition().findUpdatedConditionNamesByPolicy(policyId));
 
 			this.service    = xService;
+			this.serviceDef = daoMgr.getXXServiceDef().getById(xService.getType());
 			this.iterPolicy = asList(xPolicy).listIterator();
-			this.iterPolicyLabels = daoMgr.getXXPolicyLabelMap().findByPolicyId(policyId).listIterator();
+			List<XXPolicyLabelMap> policyLabels = daoMgr.getXXPolicyLabelMap().findByPolicyId(policyId);
+			this.iterPolicyLabels = policyLabels != null ? policyLabels.listIterator() : null;
 		}
 
 		RangerPolicy getNextPolicy() {
@@ -532,7 +538,7 @@ public class RangerPolicyRetriever {
 						ret.setVersion(xPolicy.getVersion());
 						ret.setPolicyType(xPolicy.getPolicyType() == null ? RangerPolicy.POLICY_TYPE_ACCESS : xPolicy.getPolicyType());
 						ret.setService(service.getName());
-						ret.setServiceType(service.getType().toString());
+						ret.setServiceType(serviceDef.getName());
 						ret.setZoneName(lookupCache.getSecurityZoneName(xPolicy.getZoneId()));
 						updatePolicyReferenceFields(ret);
 						getPolicyLabels(ret);
@@ -545,19 +551,21 @@ public class RangerPolicyRetriever {
 
 		private void getPolicyLabels(RangerPolicy ret) {
 			List<String> xPolicyLabels = new ArrayList<String>();
-			while (iterPolicyLabels.hasNext()) {
-				XXPolicyLabelMap xPolicyLabel = iterPolicyLabels.next();
-				if (xPolicyLabel.getPolicyId().equals(ret.getId())) {
-					String policyLabel = lookupCache.getPolicyLabelName(xPolicyLabel.getPolicyLabelId());
-					if (policyLabel != null) {
-						xPolicyLabels.add(policyLabel);
+			if (iterPolicyLabels != null) {
+				while (iterPolicyLabels.hasNext()) {
+					XXPolicyLabelMap xPolicyLabel = iterPolicyLabels.next();
+					if (xPolicyLabel.getPolicyId().equals(ret.getId())) {
+						String policyLabel = lookupCache.getPolicyLabelName(xPolicyLabel.getPolicyLabelId());
+						if (policyLabel != null) {
+							xPolicyLabels.add(policyLabel);
+						}
+						ret.setPolicyLabels(xPolicyLabels);
+					} else {
+						if (iterPolicyLabels.hasPrevious()) {
+							iterPolicyLabels.previous();
+						}
+						break;
 					}
-					ret.setPolicyLabels(xPolicyLabels);
-				} else {
-					if (iterPolicyLabels.hasPrevious()) {
-						iterPolicyLabels.previous();
-					}
-					break;
 				}
 			}
 		}
