@@ -42,6 +42,7 @@ public class RangerConstants extends RangerCommonEnums {
 	public final static String GROUP_PUBLIC = "public";
     public final static String ROLE_ADMIN_AUDITOR = "ROLE_ADMIN_AUDITOR";
     public final static String ROLE_KEY_ADMIN_AUDITOR = "ROLE_KEY_ADMIN_AUDITOR";
+    public final static String ROLE_FIELD = "Roles";
 	// Action constants
 	public final static String ACTION_EDIT = "edit";
 	public final static String ACTION_CHANGE = "change";
@@ -187,5 +188,6 @@ public class RangerConstants extends RangerCommonEnums {
 		ALLOW_WRITE,
 		ALLOW_DELETE
 	}
-
+	//HTTP STATUS code for authentication timeout
+		public static final int SC_AUTHENTICATION_TIMEOUT = 419;
 }
