@@ -21,7 +21,7 @@ package org.apache.ranger.common;
 
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
-import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
+import org.apache.ranger.authorization.hadoop.config.RangerAdminConfig;
 import org.apache.ranger.plugin.model.RangerPolicy;
 import org.apache.ranger.plugin.store.ServiceStore;
 
@@ -61,7 +61,9 @@ public class RangerServicePoliciesCache {
 	}
 
 	private RangerServicePoliciesCache() {
-		waitTimeInSeconds = RangerConfiguration.getInstance().getInt("ranger.admin.policy.download.cache.max.waittime.for.update", MAX_WAIT_TIME_FOR_UPDATE);
+		RangerAdminConfig config = RangerAdminConfig.getInstance();
+
+		waitTimeInSeconds = config.getInt("ranger.admin.policy.download.cache.max.waittime.for.update", MAX_WAIT_TIME_FOR_UPDATE);
 	}
 
 	public void dump() {
@@ -130,7 +132,7 @@ public class RangerServicePoliciesCache {
 		}
 
 		if (LOG.isDebugEnabled()) {
-			LOG.debug("<== RangerServicePoliciesCache.getServicePolicies(" + serviceName + ", " + serviceId + ", " + lastKnownVersion + ", " + needsBackwardCompatibility + "): count=" + ((ret == null || ret.getPolicies() == null) ? 0 : ret.getPolicies().size()));
+			LOG.debug("<== RangerServicePoliciesCache.getServicePolicies(" + serviceName + ", " + serviceId + ", " + lastKnownVersion + ", " + needsBackwardCompatibility + "): ret:[" + ret + "]");
 		}
 
 		return ret;
@@ -206,7 +208,7 @@ public class RangerServicePoliciesCache {
 						ServicePolicies servicePoliciesForDeltas  = this.deltaCache != null ? this.deltaCache.getServicePolicyDeltasFromVersion(lastKnownVersion) : null;
 
 						if (servicePoliciesForDeltas == null) {
-							servicePoliciesForDeltas  = serviceStore.getOnlyServicePolicyDeltas(serviceName, lastKnownVersion);
+							servicePoliciesForDeltas  = serviceStore.getServicePolicyDeltas(serviceName, lastKnownVersion);
 							isDeltaCacheReinitialized = true;
 						}
 						if (servicePoliciesForDeltas != null && servicePoliciesForDeltas.getPolicyDeltas() != null) {
@@ -218,7 +220,7 @@ public class RangerServicePoliciesCache {
 							}
 							ret = servicePoliciesForDeltas;
 						} else {
-							LOG.warn("Deltas were requested, but could not get them!! lastKnownVersion:[" + lastKnownVersion + "]; Returning cached ServicePolicies:[" + (servicePolicies != null ? servicePolicies.getPolicyVersion() : -1L) + "]");
+							LOG.warn("Deltas were requested for service:[" + serviceName + "], but could not get them!! lastKnownVersion:[" + lastKnownVersion + "]; Returning cached ServicePolicies:[" + (servicePolicies != null ? servicePolicies.getPolicyVersion() : -1L) + "]");
 
 							this.deltaCache = null;
 							ret = this.servicePolicies;
@@ -268,9 +270,10 @@ public class RangerServicePoliciesCache {
 				}
 
 				final long            startTimeMs           = System.currentTimeMillis();
-				final ServicePolicies servicePoliciesFromDb = serviceStore.getServicePolicyDeltasOrPolicies(serviceName, cachedServicePoliciesVersion);
 				final long            dbLoadTime            = System.currentTimeMillis() - startTimeMs;
 
+				ServicePolicies       servicePoliciesFromDb = serviceStore.getServicePolicyDeltasOrPolicies(serviceName, cachedServicePoliciesVersion);
+
 				if (dbLoadTime > longestDbLoadTimeInMs) {
 					longestDbLoadTimeInMs = dbLoadTime;
 				}
@@ -303,21 +306,35 @@ public class RangerServicePoliciesCache {
 
 						final List<RangerPolicy> policies = servicePolicies.getPolicies() == null ? new ArrayList<>() : servicePolicies.getPolicies();
 						final List<RangerPolicy> newPolicies = RangerPolicyDeltaUtil.applyDeltas(policies, servicePoliciesFromDb.getPolicyDeltas(), servicePolicies.getServiceDef().getName());
-						servicePolicies.setPolicies(newPolicies);
 
-						// Rebuild tag-policies from original tag-policies and deltas
-						if (servicePoliciesFromDb.getTagPolicies() != null) {
-							if (LOG.isDebugEnabled()) {
-								LOG.debug("This service has associated tag service. Will compute tagPolicies from corresponding policy-deltas");
-							}
-
-							final List<RangerPolicy> tagPolicies = (servicePolicies.getTagPolicies() == null || CollectionUtils.isEmpty(servicePolicies.getTagPolicies().getPolicies())) ? new ArrayList<>() : servicePolicies.getTagPolicies().getPolicies();
-							final List<RangerPolicy> newTagPolicies = RangerPolicyDeltaUtil.applyDeltas(tagPolicies, servicePoliciesFromDb.getPolicyDeltas(), servicePoliciesFromDb.getTagPolicies().getServiceDef().getName());
-							servicePolicies.getTagPolicies().setPolicies(newTagPolicies);
+						boolean isSanityCheckPassed = checkAndLoadCompleteSetOfPolicies(serviceName, serviceStore, newPolicies);
 
+						if (!isSanityCheckPassed) {
+							isCacheReloadedByDQEvent = true;
 						} else {
-							if (LOG.isDebugEnabled()) {
-								LOG.debug("This service has no associated tag service");
+							servicePolicies.setPolicies(newPolicies);
+
+							// Rebuild tag-policies from original tag-policies and deltas
+							if (servicePoliciesFromDb.getTagPolicies() != null) {
+								if (LOG.isDebugEnabled()) {
+									LOG.debug("This service has associated tag service. Will compute tagPolicies from corresponding policy-deltas");
+								}
+
+								final List<RangerPolicy> tagPolicies = (servicePolicies.getTagPolicies() == null || CollectionUtils.isEmpty(servicePolicies.getTagPolicies().getPolicies())) ? new ArrayList<>() : servicePolicies.getTagPolicies().getPolicies();
+								final List<RangerPolicy> newTagPolicies = RangerPolicyDeltaUtil.applyDeltas(tagPolicies, servicePoliciesFromDb.getPolicyDeltas(), servicePoliciesFromDb.getTagPolicies().getServiceDef().getName());
+
+								isSanityCheckPassed = checkAndLoadCompleteSetOfPolicies(serviceName, servicePoliciesFromDb.getTagPolicies().getServiceName(), serviceStore, newTagPolicies);
+
+								if (!isSanityCheckPassed) {
+									isCacheReloadedByDQEvent = true;
+								} else {
+									servicePolicies.getTagPolicies().setPolicies(newTagPolicies);
+								}
+
+							} else {
+								if (LOG.isDebugEnabled()) {
+									LOG.debug("This service has no associated tag service");
+								}
 							}
 						}
 					}
@@ -344,6 +361,53 @@ public class RangerServicePoliciesCache {
 			return isCacheReloadedByDQEvent;
 		}
 
+		private boolean checkAndLoadCompleteSetOfPolicies(String serviceName, ServiceStore serviceStore, List<RangerPolicy> policiesFromIncrementalComputation) {
+			boolean ret = checkCacheSanity(serviceName, serviceStore, policiesFromIncrementalComputation);
+
+			if (!ret) {
+				loadCompleteSetOfPolicies(serviceName, serviceStore);
+				LOG.warn("Policies loaded from db for service:" + serviceName + "] are [" + servicePolicies + "]");
+				LOG.warn("Policies in cache for service:[" + serviceName + "] are [" + policiesFromIncrementalComputation + "]");
+
+			}
+
+			return ret;
+		}
+
+		private boolean checkAndLoadCompleteSetOfPolicies(String serviceName, String tagServiceName, ServiceStore serviceStore, List<RangerPolicy> policiesFromIncrementalComputation) {
+			boolean ret = checkCacheSanity(tagServiceName, serviceStore, policiesFromIncrementalComputation);
+
+			if (!ret) {
+				loadCompleteSetOfPolicies(serviceName, serviceStore);
+				LOG.warn("Policies loaded from db for service:" + serviceName + "] are [" + servicePolicies + "]");
+				LOG.warn("Tag Policies in cache for tag-service:[" + tagServiceName + "] are [" + policiesFromIncrementalComputation + "]");
+			}
+
+			return ret;
+		}
+
+		private boolean checkCacheSanity(String serviceName, ServiceStore serviceStore, List<RangerPolicy> policiesFromIncrementalComputation) {
+			final boolean ret;
+			long dbPoliciesCount = serviceStore.getPoliciesCount(serviceName);
+			long cachedPoliciesCount = (CollectionUtils.isEmpty(policiesFromIncrementalComputation) ? 0 : policiesFromIncrementalComputation.size());
+			ret = dbPoliciesCount == cachedPoliciesCount;
+			if (!ret) {
+				LOG.warn("For service:[" + serviceName + "]: dbPoliciesCount:[" + dbPoliciesCount + "], cachedPoliciesCount:[" + cachedPoliciesCount + "]");
+			}
+			return ret;
+
+		}
+
+		private void loadCompleteSetOfPolicies(String serviceName, ServiceStore serviceStore) {
+			LOG.warn("Something went wrong doing incremental policies for service:[" + serviceName + "]!! Loading all policies!!");
+			try {
+				servicePolicies = serviceStore.getServicePolicies(serviceName, -1L);
+				pruneUnusedAttributes();
+			} catch (Exception ex) {
+				LOG.warn("Could not get policies from database");
+			}
+		}
+
 		private void pruneUnusedAttributes() {
 			if (servicePolicies != null) {
 				pruneUnusedPolicyAttributes(servicePolicies.getPolicies());
